# -*- encoding: utf-8 -*-
# Pilas engine - A video game framework.
#
# Copyright 2010 - Hugo Ruscitti
# License: LGPLv3 (see http://www.gnu.org/licenses/lgpl.html)
#
# Website - http://www.pilas-engine.com.ar

import pilas
import os

def cargar(ruta):
    """Intenta cargar la imagen indicada por el argumento ``ruta``.

    Por ejemplo::

        import pilas

        imagen = pilas.imagenes.cargar("mi_archivo.png")

    En caso de éxito retorna el objeto Image, que se puede asignar
    a un Actor.

    El directorio de búsqueda de la imagen sigue el siguiente orden:

        * primero busca en el directorio actual.
        * luego en 'data'.
        * por último en el directorio estándar de la biblioteca.

    En caso de error genera una excepción de tipo IOError.
    """

    ruta = pilas.utils.obtener_ruta_al_recurso(ruta)
    return pilas.motor.cargar_imagen(ruta)


Grilla = pilas.motor.Grilla



# -*- encoding: utf-8 -*-
# pilas engine - a video game framework.
#
# copyright 2010 - hugo ruscitti
# license: lgplv3 (see http://www.gnu.org/licenses/lgpl.html)
#
# website - http://www.pilas-engine.com.ar


import pilas
import pilas.actores


class ModoEjecucion:
    """Representa un modo de ejecucion del mundo.

    Este objeto representa la estrategia que el mundo tiene
    en un momento particular. Esta abstraccion sirve para
    implementar los modos 'pausa' y 'depuracion' principalmente.
    """

    def __init__(self, mundo):
        self.mundo = mundo

    def alternar_modo_depuracion(self):
        raise Exception("Tiene que redefinir este metodo.")

    def alternar_pausa(self):
        raise Exception("Tiene que redefinir este metodo.")


class ModoEjecucionNormal(ModoEjecucion):
    """Representa al mundo cuando el usuario está jugando normalmente."""

    def actualizar_simuladores(self):
        self.mundo.tweener.update(16)
        self.mundo.tasks.update(16/1000.0)
        pilas.fisica.fisica.actualizar()

    def actualizar_actores(self):
        for actor in pilas.actores.todos:
            actor.actualizar_comportamientos()
            actor.actualizar_habilidades()
            actor.actualizar()

    def dibujar_actores(self):
        # Separo el dibujado de los actores porque la lista puede cambiar
        # dutante la actualizacion de actores (por ejemplo si uno se elimina).
        for actor in pilas.actores.todos:
            actor.dibujar(self.mundo.ventana)

    def emitir_evento_actualizar(self):
        self.mundo.control.actualizar()
        pilas.eventos.actualizar.send("bucle")

    def salir(self):
        pass

    def analizar_colisiones(self):
        pilas.colisiones.verificar_colisiones()

    def alternar_modo_depuracion(self):
        self.mundo.definir_modo_ejecucion(ModoEjecucionDepuracion(self.mundo))

    def alternar_pausa(self):
        self.mundo.definir_modo_ejecucion(ModoEjecucionPausado(self.mundo))

 
class ModoEjecucionPausado(ModoEjecucionNormal):
    """Representa al mundo pero en pausa, donde se muestra la imagen sin moverse.

    En este modo puedes pulsar la tecla barra espaciadora para avanzar
    un solo cuadro de animación o la tecla derecha para avanzar rápidamente
    en la simulación.

    """

    def __init__(self, m):
        ModoEjecucionNormal.__init__(self, m)
        self.icono = pilas.actores.Actor("icono_pausa.png")
        self.icono.z = -100
        # Hace que no funcione la repreticion de tecla para espacio
        # (asi se puede avanzar la simulacion paso a paso).
        self.pulso_avanzar = False

    def actualizar_simuladores(self):
        pass

    def actualizar_actores(self):
        pass

    def emitir_evento_actualizar(self):
        self.mundo.control.actualizar()

        # Permite avanzar cuadros de animacion rápidamente
        # pulsando la tecla derecha.
        if self.mundo.control.derecha:
            self.actualizar_un_paso()

        # Permite avanzar de a un solo cuadro pulsando
        # la barra espaciadora.
        if self.mundo.control.boton and not self.pulso_avanzar:
            self.actualizar_un_paso()
            self.pulso_avanzar = True
        else:
            if not self.mundo.control.boton:
                self.pulso_avanzar = False


    def actualizar_un_paso(self):
        ModoEjecucionNormal.actualizar_actores(self)
        pilas.eventos.actualizar.send("bucle")

    def alternar_modo_depuracion(self):
        self.mundo.definir_modo_ejecucion(ModoEjecucionDepuracionPausado(self.mundo))

    def alternar_pausa(self):
        self.mundo.definir_modo_ejecucion(ModoEjecucionNormal(self.mundo))

    def salir(self):
        self.icono.eliminar()


class ModoEjecucionDepuracion(ModoEjecucionNormal):
    """Es el modo de ejecucion normal pero mostrando informacion para desarrolladores."""

    def __init__(self, m):
        import pilas.actores
        ModoEjecucionNormal.__init__(self, m)
        self.eje = pilas.actores.Ejes()
        pilas.eventos.inicia_modo_depuracion.send("ModoEjecucionDepuracion")

    def salir(self):
        self.eje.eliminar()

    def dibujar_actores(self):

        ModoEjecucionNormal.dibujar_actores(self)

        color_de_colision = pilas.colores.verde_transparente
        color_de_punto_de_control = pilas.colores.rojo_transparente
        color_borde = pilas.colores.gris_transparente

        for actor in pilas.actores.todos:
            if actor.radio_de_colision:
                self.pintar_radio_de_colision_del_actor(actor, color_de_colision, color_borde)

            self.pintar_punto_de_control_del_actor(actor, color_de_punto_de_control)

            # intenta dibujar el numero de cuadro que usa el actor
            # si es que tiene una animacion asignada.

            '''
            try:
                cuadro = actor.animacion.obtener_cuadro()
                self.pintar_numero(cuadro, actor.x, actor.y)
            except AttributeError, e:
                pass
            '''

    def pintar_numero(self, numero, x, y):
        numero = pilas.actores.Texto(str(numero))
        numero.x = x
        numero.y = y

        self.mundo.ventana.Draw(numero)
        numero.eliminar()

    def pintar_radio_de_colision_del_actor(self, actor, color, color_borde):
        x, y = actor.x, actor.y
        radio = actor.radio_de_colision * 2
        pilas.motor.dibujar_circulo(x, y, radio, color, color_borde)

    def pintar_punto_de_control_del_actor(self, actor, color):
        x, y = actor.x, actor.y
        pilas.motor.dibujar_circulo(x, y, 3, color, color)

    def alternar_modo_depuracion(self):
        self.mundo.definir_modo_ejecucion(ModoEjecucionNormal(self.mundo))

    def alternar_pausa(self):
        self.mundo.definir_modo_ejecucion(ModoEjecucionDepuracionPausado(self.mundo))

    def salir(self):
        pilas.eventos.sale_modo_depuracion.send("ModoEjecucionNormal")
        self.eje.eliminar()

    def actualizar_actores(self):
        pilas.eventos.actualiza_modo_depuracion.send("ModoEjecucionDepuracion")
        ModoEjecucionNormal.actualizar_actores(self)


class ModoEjecucionDepuracionPausado(ModoEjecucionPausado, ModoEjecucionDepuracion):

    def __init__(self, m):
        ModoEjecucionPausado.__init__(self, m)
        ModoEjecucionDepuracion.__init__(self, m)

    def dibujar_actores(self):
        ModoEjecucionDepuracion.dibujar_actores(self)

    def salir(self):
        ModoEjecucionPausado.salir(self)
        ModoEjecucionDepuracion.salir(self)

    def actualizar_actores(self):
        pass

    def alternar_modo_depuracion(self):
        self.mundo.definir_modo_ejecucion(ModoEjecucionPausado(self.mundo))

    def alternar_pausa(self):
        self.mundo.definir_modo_ejecucion(ModoEjecucionDepuracion(self.mundo))



# -*- encoding: utf-8 -*-
# Pilas engine - A video game framework.
#
# Copyright 2010 - Hugo Ruscitti
# License: LGPLv3 (see http://www.gnu.org/licenses/lgpl.html)
#
# Website - http://www.pilas-engine.com.ar

import pilas
import os


def cargar(ruta):
    """Carga un sonido para reproducir, donde el argumento ``ruta`` indica cual es el archivo.

    Por ejemplo::

        import pilas

        risa = pilas.sonidos.cargar("risa.ogg")

    En caso de éxito retorna el objeto Sound, que se puede
    reproducir usando el método ``reproducir()``, por ejemplo::

        risa.reproducir()

    El directorio de búsqueda del sonido sigue el siguiente orden:

        * primero busca en el directorio actual.
        * luego en 'data'.
        * por último en el directorio estándar de la biblioteca.

    En caso de error genera una excepción de tipo IOError.
    """
    ruta = pilas.utils.obtener_ruta_al_recurso(ruta)
    return pilas.motor.cargar_sonido(ruta)



# -*- encoding: utf-8 -*-
# Pilas engine - A video game framework.
#
# Copyright 2010 - Hugo Ruscitti
# License: LGPLv3 (see http://www.gnu.org/licenses/lgpl.html)
#
# Website - http://www.pilas-engine.com.ar

import pilas
from pilas.actores import Actor
from pilas.comportamientos import Comportamiento

VELOCIDAD = 4

class Pingu(Actor):
    "Representa una tortuga que se mueve por la pantalla como la tortuga de Logo."

    def __init__(self, x=0, y=0):
        Actor.__init__(self, x=x, y=y)
        self.animacion = pilas.imagenes.Grilla("pingu.png", 10)
        self.definir_cuadro(4)
        self.hacer(Esperando())
        self.radio_de_colision = 30

    def definir_cuadro(self, indice):
        self.animacion.definir_cuadro(indice)
        self.animacion.asignar(self)


class Esperando(Comportamiento):
    "Un actor en posicion normal o esperando a que el usuario pulse alguna tecla."

    def iniciar(self, receptor):
        self.receptor = receptor
        self.receptor.definir_cuadro(4)

    def actualizar(self):
        if pilas.mundo.control.izquierda:
            self.receptor.hacer(Caminando())
        elif pilas.mundo.control.derecha:
            self.receptor.hacer(Caminando())

        if pilas.mundo.control.arriba:
            self.receptor.hacer(Saltando())


class Caminando(Comportamiento):

    def __init__(self):
        self.cuadros = [5, 5, 6, 6, 7, 7, 8, 8, 9, 9]
        self.paso = 0

    def actualizar(self):
        self.avanzar_animacion()

        if pilas.mundo.control.izquierda:
            self.receptor.x -= VELOCIDAD
        elif pilas.mundo.control.derecha:
            self.receptor.x += VELOCIDAD
        else:
            self.receptor.hacer(Esperando())

        if pilas.mundo.control.arriba:
            self.receptor.hacer(Saltando())


    def avanzar_animacion(self):
        self.paso += 1

        if self.paso >= len(self.cuadros):
            self.paso = 0

        self.receptor.definir_cuadro(self.cuadros[self.paso])

class Saltando(Comportamiento):

    def __init__(self):
        self.dy = 10
    
    def iniciar(self, receptor):
        self.receptor = receptor
        self.receptor.definir_cuadro(0)

    def actualizar(self):
        self.receptor.y += self.dy
        self.dy -= 0.3

        if self.receptor.y < 0:
            self.receptor.y = 0
            self.receptor.hacer(Esperando())

        if pilas.mundo.control.izquierda:
            self.receptor.x -= VELOCIDAD
        elif pilas.mundo.control.derecha:
            self.receptor.x += VELOCIDAD



#!/usr/bin/python
"""
This file is part of the 'Elements' Project
Elements is a 2D Physics API for Python (supporting pybox2d)

Copyright (C) 2008, The Elements Team, <elements@linuxuser.at>

Home:  http://elements.linuxuser.at
IRC:   #elements on irc.freenode.org

Code:  http://www.assembla.com/wiki/show/elements
       svn co http://svn2.assembla.com/svn/elements                     

License:  GPLv3 | See LICENSE for the full text
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.              
"""
__version__=  '0.11'
__contact__ = '<elements@linuxuser.at>'

# Load Box2D
try:
    import Box2D as box2d
except:
    print 'Could not load the pybox2d library (Box2D).'
    print 'Please run "setup.py install" to install the dependencies.'
    print 
    print 'Alternatively, recompile pybox2d for your system and python version.'
    print "See http://code.google.com/p/pybox2d"
    exit()

# Standard Imports
from random import shuffle

# Load Elements Definitions
from locals import *

# Load Elements Modules
import tools
import drawing
import add_objects
import callbacks
import camera

# Main Class
class Elements:
    """The class which handles all interaction with the box2d engine
    """
    # Settings
    run_physics   =True           # Can pause the simulation
    element_count =0              # Element Count 
    renderer      =None           # Drawing class (from drawing.py)
    input         =INPUT_PIXELS   # Default Input in Pixels! (can change to INPUT_METERS)
    line_width    =0              # Line Width in Pixels (0 for fill)
    listener      =None
    
    screen_offset = (0, 0)        # Offset screen from world coordinate system (x, y) [meter5]
    screen_offset_pixel = (0, 0)  # Offset screen from world coordinate system (x, y) [pixel]
    
    # The internal coordination system is y+=up, x+=right
    # But it's possible to change the input coords to something else,
    # they will then be translated on input
    inputAxis_x_left = False    # positive to the right by default
    inputAxis_y_down = True     # positive to up by default

    mouseJoint = None

    def __init__(self, screen_size, gravity=(0.0,-9.0), ppm=100.0, renderer='pygame'):
        """ Init the world with boundaries and gravity, and init colors.
        
            Parameters:
              screen_size .. (w, h) -- screen size in pixels [int]
              gravity ...... (x, y) in m/s^2  [float] default: (0.0, -9.0)
              ppm .......... pixels per meter [float] default: 100.0
              renderer ..... which drawing method to use (str) default: 'pygame'

            Return: class Elements()
        """
        self.set_screenSize(screen_size)
        self.set_drawingMethod(renderer)
        
        # Create Subclasses
        self.add = add_objects.Add(self)
        self.callbacks = callbacks.CallbackHandler(self)
        self.camera = camera.Camera(self)
        
        # Set Boundaries
        self.worldAABB=box2d.b2AABB()
        self.worldAABB.lowerBound = (-100.0, -100.0)
        self.worldAABB.upperBound = (100.0, 100.0)
        
        # Gravity + Bodies will sleep on outside
        self.gravity = gravity
        self.doSleep = True
    
        # Create the World
        self.world = box2d.b2World(self.worldAABB, self.gravity, self.doSleep)

        # Init Colors        
        self.init_colors()
        
        # Set Pixels per Meter
        self.ppm = ppm

    def set_inputUnit(self, input):
        """ Change the input unit to either meter or pixels
        
            Parameters:
              input ... INPUT_METERS or INPUT_PIXELS
          
            Return: -
        """
        self.input = input
        
    def set_inputAxisOrigin(self, left=True, top=False):
        """ Change the origin of the input coordinate system axis
        
            Parameters:
              left ... True or False -- x = 0 is at the left?
              top .... True or False -- y = 0 is at the top?
          
            Return: -
        """          
        self.inputAxis_x_left = not left
        self.inputAxis_y_down = top

    def set_drawingMethod(self, m, *kw):
        """ Set a drawing method (from drawing.py)
        
            Parameters:
              m .... 'pygame' or 'cairo'
              *kw .. keywords to pass to the initializer of the drawing method

            Return: True if ok, False if no method identifier m found
        """
        try:
            self.renderer = getattr(drawing, "draw_%s" % m) (*kw)
            return True
        except AttributeError:
            return False
            
    def set_screenSize(self, size):
        """ Set the current screen size
        
            Parameters: 
              size ... (int(width), int(height)) in pixels
              
            Return: -
        """
        self.display_width, self.display_height = size

    def init_colors(self):
        """ Init self.colors with a fix set of hex colors
        
            Return: -        
        """
        self.fixed_color = None
        self.cur_color = 0
        self.colors = [
          "#737934", "#729a55", "#040404", "#1d4e29", "#ae5004", "#615c57",
          "#6795ce", "#203d61", "#8f932b"
        ]
        shuffle(self.colors)

    def set_color(self, clr):
        """ Set a fixed color for all future Elements (until reset_color() is called) 
        
            Parameters: 
              clr ... Hex '#123123' or RGB ((r), (g), (b))
              
            Return: -
        """
        self.fixed_color = clr
    
    def reset_color(self):
        """ All Elements from now on will be drawn in random colors
           
            Return: - 
        """
        self.fixed_color = None

    def get_color(self):
        """ Get a color - either the fixed one or the next from self.colors 
        
            Return: clr = ((R), (G), (B)) 
        """
        if self.fixed_color != None:
            return self.fixed_color
            
        if self.cur_color == len(self.colors): 
            self.cur_color = 0
            shuffle(self.colors)
    
        clr = self.colors[self.cur_color]
        if clr[0] == "#":
            clr = tools.hex2rgb(clr)
        
        self.cur_color += 1
        return clr
    
    def update(self, fps=50.0, vel_iterations=10, pos_iterations=8):
        """ Update the physics, if not paused (self.run_physics)
        
            Parameters:
              fps ............. fps with which the physics engine shall work
              vel_iterations .. velocity substeps per step for smoother simulation
              pos_iterations .. position substeps per step for smoother simulation
            
            Return: -
        """
        if self.run_physics:
            self.world.Step(1.0 / fps, vel_iterations, pos_iterations)

    def translate_coord(self, point):
        """ Flips the coordinates in another coordinate system orientation, if necessary
            (screen <> world coordinate system) 
        """
        x, y = point

        if self.inputAxis_x_left:
            x = self.display_width - x

        if self.inputAxis_y_down:
            y = self.display_height - y
            
        return (x, y)
        
    def translate_coords(self, pointlist):
        """ Flips the coordinates in another coordinate system orientation, if necessary 
            (screen <> world coordinate system) 
        """    
        p_out = []        
        for p in pointlist:
            p_out.append(self.translate_coord(p))
        return p_out

    def to_world(self, pos):
        """ Transfers a coordinate from the screen to the world coordinate system (pixels)
            - Change to the right axis orientation
            - Include the offset: screen -- world coordinate system
            - Include the scale factor (Screen coordinate system might have a scale factor)
        """
        dx, dy = self.screen_offset_pixel
        
        x = pos[0] / self.camera.scale_factor
        y = pos[1] / self.camera.scale_factor
        
        x, y = self.translate_coord((round(x), round(y)))
        return (x+dx, y+dy) 
        
    def to_screen(self, pos):
        """ Transfers a coordinate from the world to the screen coordinate system (pixels)
            and by the screen offset
        """
        dx, dy = self.screen_offset_pixel
        x = pos[0] - dx
        y = pos[1] - dy
        
        sx, sy = self.translate_coord((x, y))
        return (sx * self.camera.scale_factor, sy * self.camera.scale_factor)
                         
    def meter_to_screen(self, i):
        return i * self.ppm * self.camera.scale_factor
        
    def get_bodies_at_pos(self, search_point, include_static=False, area=0.01):
        """ Check if given point (screen coordinates) is inside any body.
            If yes, return all found bodies, if not found return False
        """
        sx, sy = self.to_world(search_point)
        sx /= self.ppm
        sy /= self.ppm

        f = area/self.camera.scale_factor

        AABB=box2d.b2AABB()
        AABB.lowerBound = (sx-f, sy-f)
        AABB.upperBound = (sx+f, sy+f)

        amount, shapes = self.world.Query(AABB, 2)

        if amount == 0:
            return False
        else:
            bodylist = []
            for s in shapes:
                body = s.GetBody()
                if not include_static:
                    if body.IsStatic() or body.GetMass() == 0.0:
                        continue
                        
                if s.TestPoint(body.GetXForm(), (sx, sy)):
                    bodylist.append(body)

            return bodylist
    
    def draw(self):
        """ If a drawing method is specified, this function passes the objects
            to the module in pixels.
            
            Return: True if the objects were successfully drawn
              False if the renderer was not set or another error occurred
        """
        self.callbacks.start(CALLBACK_DRAWING_START)
        
        # No need to run through the loop if there's no way to draw        
        if not self.renderer: 
            return False

        if self.camera.track_body:
            # Get Body Center
            p1 = self.camera.track_body.GetWorldCenter()   
            
            # Center the Camera There, False = Don't stop the tracking
            self.camera.center(self.to_screen((p1.x*self.ppm, p1.y*self.ppm)), stopTrack=False) 
            
        # Walk through all known elements
        self.renderer.start_drawing()
        
        for body in self.world.bodyList:
            xform = body.GetXForm()
            shape = body.GetShapeList()
            angle = body.GetAngle()
            
            if shape:
                userdata = body.GetUserData()
                clr = userdata['color']
                
            for shape in body.shapeList:                                
                type = shape.GetType()
                                
                if type == box2d.e_circleShape:
                    position = box2d.b2Mul(xform, shape.GetLocalPosition())
                    
                    pos = self.to_screen((position.x*self.ppm, position.y*self.ppm))                    
                    self.renderer.draw_circle(clr, pos, self.meter_to_screen(shape.radius), angle)

                elif type == box2d.e_polygonShape:
                    points = []
                    for v in shape.vertices:
                        pt = box2d.b2Mul(xform, v)
                        x, y = self.to_screen((pt.x*self.ppm, pt.y*self.ppm))
                        points.append([x, y])

                    self.renderer.draw_polygon(clr, points)
                   
                else:
                    print "  unknown shape type:%d" % shape.GetType()
    

        for joint in self.world.jointList:
            p2 = joint.GetAnchor1()
            p2 = self.to_screen((p2.x*self.ppm, p2.y*self.ppm))
            
            p1 = joint.GetAnchor2()
            p1 = self.to_screen((p1.x*self.ppm, p1.y*self.ppm))
            
            if p1 == p2:
                self.renderer.draw_circle((255,255,255), p1, 2, 0)
            else:
                self.renderer.draw_lines((0,0,0), False, [p1, p2], 3)

        self.callbacks.start(CALLBACK_DRAWING_END)
        self.renderer.after_drawing()
        
        return True


    def mouse_move(self, pos):
        pos = self.to_world(pos)
        x, y = pos
        x /= self.ppm
        y /= self.ppm
                        
        if self.mouseJoint:
            self.mouseJoint.SetTarget((x,y))

    def pickle_save(self, fn, additional_vars={}):
        import cPickle as pickle
        self.add.remove_mouseJoint()
    
        if not additional_vars and hasattr(self, '_pickle_vars'):
            additional_vars=dict((var, getattr(self, var)) for var in self._pickle_vars)

        save_values = [self.world, box2d.pickle_fix(self.world, additional_vars, 'save')]

        try:
            pickle.dump(save_values, open(fn, 'wb'))
        except Exception, s:
            print 'Pickling failed: ', s
            return

        print 'Saved to %s' % fn

    def pickle_load(self, fn, set_vars=True, additional_vars=[]):
        """
        Load the pickled world in file fn.
        additional_vars is a dictionary to be populated with the
        loaded variables.
        """
        import cPickle as pickle
        try:
            world, variables = pickle.load(open(fn, 'rb'))
            world = world._pickle_finalize()
            variables  = box2d.pickle_fix(world, variables, 'load')
        except Exception, s:
            print 'Error while loading world: ', s
            return
        
        self.world = world
        
        if set_vars:
            # reset the additional saved variables:
            for var, value in variables.items():
                if hasattr(self, var):
                    setattr(self, var, value)
                else:
                    print 'Unknown property %s=%s' % (var, value)

        print 'Loaded from %s' % fn

        return variables

    def json_save(self, path, additional_vars = {}):
        import cjson
        worldmodel = {}

        save_id_index = 1
        self.world.GetGroundBody().userData = {"saveid" : 0}

        bodylist = []
        for body in self.world.GetBodyList():
            if not body == self.world.GetGroundBody():
                body.userData["saveid"] = save_id_index #set temporary data
                save_id_index+=1
                shapelist = body.GetShapeList()
                modelbody = {}
                modelbody['position'] = body.position.tuple()
                modelbody['dynamic'] = body.IsDynamic()
                modelbody['userData'] = body.userData
                modelbody['angle'] = body.angle
                modelbody['angularVelocity'] = body.angularVelocity
                modelbody['linearVelocity'] = body.linearVelocity.tuple()
                if shapelist and len(shapelist) > 0:
                    shapes = []
                    for shape in shapelist:
                        modelshape = {}
                        modelshape['density'] = shape.density
                        modelshape['restitution'] = shape.restitution
                        modelshape['friction'] = shape.friction
                        shapename = shape.__class__.__name__
                        if  shapename == "b2CircleShape":
                            modelshape['type'] = 'circle'
                            modelshape['radius'] = shape.radius
                            modelshape['localPosition'] = shape.localPosition.tuple()
                        if shapename == "b2PolygonShape":
                            modelshape['type'] = 'polygon'
                            modelshape['vertices'] = shape.vertices
                        shapes.append(modelshape)
                    modelbody['shapes'] = shapes

                bodylist.append(modelbody)

        worldmodel['bodylist'] = bodylist

        jointlist = []

        for joint in self.world.GetJointList():
            modeljoint = {}

            if joint.__class__.__name__ == "b2RevoluteJoint":
                modeljoint['type'] = 'revolute'
                modeljoint['anchor'] = joint.GetAnchor1().tuple()
                modeljoint['enableMotor'] = joint.enableMotor
                modeljoint['motorSpeed'] = joint.motorSpeed
                modeljoint['maxMotorTorque'] = joint.maxMotorTorque
            elif joint.__class__.__name__ == "b2DistanceJoint":
                modeljoint['type'] = 'distance'
                modeljoint['anchor1'] = joint.GetAnchor1().tuple()
                modeljoint['anchor2'] = joint.GetAnchor2().tuple()

            modeljoint['body1'] = joint.body1.userData['saveid']
            modeljoint['body2'] = joint.body2.userData['saveid']
            modeljoint['collideConnected'] = joint.collideConnected
            modeljoint['userData'] = joint.userData


            jointlist.append(modeljoint)

        worldmodel['jointlist'] = jointlist

        controllerlist = []
        worldmodel['controllerlist'] = controllerlist

        worldmodel['additional_vars'] = additional_vars

        f = open(path,'w')
        f.write(cjson.encode(worldmodel))
        f.close()

        for body in self.world.GetBodyList():
            del body.userData['saveid'] #remove temporary data

    def json_load(self, path, additional_vars = {}):
        import cjson

        self.world.GetGroundBody().userData = {"saveid" : 0}

        f = open(path, 'r')
        worldmodel = cjson.decode(f.read())
        f.close()
        #clean world
        for joint in self.world.GetJointList():
            self.world.DestroyJoint(joint)
        for body in self.world.GetBodyList():
            if body != self.world.GetGroundBody():
                self.world.DestroyBody(body)

        #load bodys
        for body in worldmodel['bodylist']:
            bodyDef = box2d.b2BodyDef()
            bodyDef.position = body['position']
            bodyDef.userData = body['userData']
            bodyDef.angle = body['angle']
            newBody = self.world.CreateBody(bodyDef)
            #_logger.debug(newBody)
            newBody.angularVelocity = body['angularVelocity']
            newBody.linearVelocity = body['linearVelocity']
            if body.has_key('shapes'):
                for shape in body['shapes']:
                    if shape['type'] == 'polygon':
                        polyDef = box2d.b2PolygonDef()
                        polyDef.setVertices(shape['vertices'])
                        polyDef.density = shape['density']
                        polyDef.restitution = shape['restitution']
                        polyDef.friction = shape['friction']
                        newBody.CreateShape(polyDef)
                    if shape['type'] == 'circle':
                        circleDef = box2d.b2CircleDef()
                        circleDef.radius = shape['radius']
                        circleDef.density = shape['density']
                        circleDef.restitution = shape['restitution']
                        circleDef.friction = shape['friction']
                        circleDef.localPosition  = shape['localPosition']
                        newBody.CreateShape(circleDef)
                newBody.SetMassFromShapes()

        for joint in worldmodel['jointlist']:
            if joint['type'] == 'distance':
                jointDef = box2d.b2DistanceJointDef()
                body1 = self.getBodyWithSaveId(joint['body1'])
                anch1 = joint['anchor1']
                body2 = self.getBodyWithSaveId(joint['body2'])
                anch2 = joint['anchor2']
                jointDef.collideConnected = joint['collideConnected']
                jointDef.Initialize(body1,body2,anch1,anch2)
                jointDef.SetUserData(joint['userData'])
                self.world.CreateJoint(jointDef)
            if joint['type'] == 'revolute':
                jointDef = box2d.b2RevoluteJointDef()
                body1 = self.getBodyWithSaveId(joint['body1'])
                body2 = self.getBodyWithSaveId(joint['body2'])
                anchor = joint['anchor']
                jointDef.Initialize(body1,body2,anchor)
                jointDef.SetUserData(joint['userData'])
                jointDef.enableMotor = joint['enableMotor']
                jointDef.motorSpeed = joint['motorSpeed']
                jointDef.maxMotorTorque = joint['maxMotorTorque']
                self.world.CreateJoint(jointDef)

        for (k,v) in worldmodel['additional_vars'].items():
            additional_vars[k] = v

        for body in self.world.GetBodyList():
            del body.userData['saveid'] #remove temporary data

    def getBodyWithSaveId(self,saveid):
        for body in self.world.GetBodyList():
            if body.userData['saveid'] == saveid:
                return body



# -*- encoding: utf-8 -*-
# Pilas engine - A video game framework.
#
# Copyright 2010 - Hugo Ruscitti
# License: LGPLv3 (see http://www.gnu.org/licenses/lgpl.html)
#
# Website - http://www.pilas-engine.com.ar

import pilas
from pilas.actores import Actor
from pilas.comportamientos import Comportamiento

VELOCIDAD = 1000


class Martian(Actor):

    def __init__(self, x=0, y=0):
        Actor.__init__(self, x=x, y=y)
        self.animacion = pilas.imagenes.Grilla("marcianitos/martian.png", 12)
        self.definir_cuadro(0)
        self.hacer(Esperando())
        self.aprender(pilas.habilidades.PisaPlataformas)

    def definir_cuadro(self, indice):
        self.animacion.definir_cuadro(indice)
        self.animacion.asignar(self)
        self.definir_centro(32, 115)

    def mover(self, x, y):
        pilas.fisica.fisica.empujar_figura(self.habilidades[0].figura, x, y)

class Esperando(Comportamiento):
    "Un actor en posicion normal o esperando a que el usuario pulse alguna tecla."

    def iniciar(self, receptor):
        self.receptor = receptor
        self.receptor.definir_cuadro(0)

    def actualizar(self):
        if pilas.mundo.control.izquierda:
            self.receptor.espejado = True
            self.receptor.hacer(Caminando())
        elif pilas.mundo.control.derecha:
            self.receptor.espejado = False
            self.receptor.hacer(Caminando())

        if pilas.mundo.control.arriba:
            self.receptor.hacer(Saltando())


class Caminando(Comportamiento):

    def __init__(self):
        self.cuadros = [1, 1, 1, 2, 2, 2]
        self.paso = 0

    def actualizar(self):
        self.avanzar_animacion()

        if pilas.mundo.control.izquierda:
            self.receptor.mover(-VELOCIDAD, 0)
        elif pilas.mundo.control.derecha:
            self.receptor.mover(VELOCIDAD, 0)
        else:
            self.receptor.hacer(Esperando())

        if pilas.mundo.control.arriba:
            self.receptor.hacer(Saltando())

    def avanzar_animacion(self):
        self.paso += 1

        if self.paso >= len(self.cuadros):
            self.paso = 0

        self.receptor.definir_cuadro(self.cuadros[self.paso])

class Saltando(Comportamiento):

    def iniciar(self, receptor):
        self.receptor = receptor
        self.receptor.definir_cuadro(3)
        self.receptor.mover(0, 4000)

    def actualizar(self):
        #self.receptor.y += self.dy
        #self.dy -= 0.3

        #if self.receptor.y < 0:
        #    self.receptor.y = 0
        #    self.receptor.hacer(Esperando())

        if pilas.mundo.control.izquierda:
            self.receptor.espejado = True
            self.receptor.mover(-VELOCIDAD, 0)
        elif pilas.mundo.control.derecha:
            self.receptor.espejado = False
            self.receptor.mover(+VELOCIDAD, 0)



import pilas
import sys

pilas.iniciar()
pizarra = pilas.actores.Pizarra()

grilla = pilas.imagenes.Grilla("pingu.png", 10)
pizarra.pintar_grilla(grilla, 0, 0)

grilla.definir_cuadro(2)
pizarra.pintar_grilla(grilla, 100, 100)

grilla.definir_cuadro(3)
pizarra.pintar_grilla(grilla, 200, 200)

pilas.ejecutar()



import pilas


pilas.iniciar()

imagen_explosion = pilas.imagenes.Grilla("explosion.png", 7)
sonido_explosion = pilas.sonidos.cargar("explosion.wav")

def crear_explosion(evento):
    explosion = pilas.actores.Animacion(imagen_explosion)
    explosion.x = evento.x
    explosion.y = evento.y
    sonido_explosion.reproducir()


pilas.eventos.click_de_mouse.connect(crear_explosion)
pilas.avisar("Pulse el boton de mouse para crear explosiones.")
pilas.ejecutar()



"""
This file is part of the 'Elements' Project
Elements is a 2D Physics API for Python (supporting Box2D2)

Copyright (C) 2008, The Elements Team, <elements@linuxuser.at>

Home:  http://elements.linuxuser.at
IRC:   #elements on irc.freenode.org

Code:  http://www.assembla.com/wiki/show/elements
       svn co http://svn2.assembla.com/svn/elements                     

License:  GPLv3 | See LICENSE for the full text
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.              
"""
from functools import partial

from math import fabs
from math import sqrt
from math import atan2
from math import degrees
from math import acos

from locals import *
from elements import box2d

def calc_center(points):
    """ Calculate the center of a polygon
    
        Return: The center (x,y)
    """
    tot_x, tot_y = 0,0
    for p in points:
        tot_x += p[0]
        tot_y += p[1]
    n = len(points)
    return (tot_x/n, tot_y/n)
    
def poly_center_vertices(pointlist):
    """ Rearranges vectors around the center
    
        Return: pointlist ([(x, y), ...])
    """    
    poly_points_center = []
    center = cx, cy = calc_center(pointlist)
    
    for p in pointlist:
        x = p[0] - cx
        y = cy - p[1]
        poly_points_center.append((x, y))
    
    return poly_points_center
    
def is_line(vertices, tolerance=25.0):
    """ Check if passed vertices are a line. Done by comparing
        the angles of all vectors and check tolerance.
        
        Parameters:
          vertices ... a list of vertices (x, y)
          tolerance .. how many degrees should be allowed max to be a line
          
        Returns: True if line, False if no line
    """
    if len(vertices) <= 2:        
        return True

    # Step 1: Points -> Vectors
    p_old = vertices[0]
    alphas = []
    

    for p in vertices[1:]:
        x1, y1 = p_old
        x2, y2 = p
        p_old = p

        # Create Vector        
        vx, vy = (x2-x1, y2-y1)
        
        # Check Length
        l = sqrt((vx*vx) + (vy*vy))
        if l == 0.0: continue

        # Normalize vector
        vx /= l
        vy /= l
        
        # Append angle
        if fabs(vx) < 0.2: alpha = 90.0
        else: alpha = degrees(atan2(vy,vx))

        alphas.append(fabs(alpha))
        
    # Sort angles
    alphas.sort()
    
    # Get maximum difference
    alpha_diff = fabs(alphas[-1] - alphas[0])
    print "alpha difference:", alpha_diff
    
    if alpha_diff < tolerance:
        return True
    else:
        return False
    
def reduce_poly_by_angle(vertices, tolerance=10.0, minlen=20):
    """ This function reduces a poly by the angles of the vectors (detect lines)
        If the angle difference from one vector to the last > tolerance: use last point
        If the angle is quite the same, it's on the line
        
        Parameters:
          vertices ... a list of vertices (x, y)
          tolerance .. how many degrees should be allowed max
          
        Returns: (1) New Pointlist, (2) Soft reduced pointlist (reduce_poly())
    """
    v_last = vertices[-1]
    vertices = vxx = reduce_poly(vertices, minlen)
    
    p_new =  []
    p_new.append(vertices[0])
    
    dir = None 
    is_convex = True
    
    for i in xrange(len(vertices)-1):
        if i == 0:
            p_old = vertices[i]
            continue
        
        x1, y1 = p_old
        x2, y2 = vertices[i]
        x3, y3 = vertices[i+1]
        p_old = vertices[i]
        
        # Create Vectors
        v1x = (x2 - x1) * 1.0
        v1y = (y2 - y1) * 1.0
        
        v2x = (x3 - x2) * 1.0
        v2y = (y3 - y2) * 1.0

        # Calculate angle
        a = ((v1x * v2x) + (v1y * v2y))
        b = sqrt((v1x*v1x) + (v1y*v1y))
        c = sqrt((v2x*v2x) + (v2y*v2y))

        # No Division by 0 :)        
        if (b*c) == 0.0: continue
        
        # Get the current degrees
        # We have a bug here sometimes...
        try:
            angle = degrees(acos(a / (b*c)))
        except:
            # cos=1.0
            print "cos=", a/(b*c)
            continue
            
        # Check if inside tolerance
        if fabs(angle) > tolerance:
            p_new.append(vertices[i])        
            # print "x", 180-angle, is_left(vertices[i-1], vertices[i], vertices[i+1])
            
            # Check if convex:
            if dir == None:
                dir = is_left(vertices[i-1], vertices[i], vertices[i+1])
            else:
                if dir != is_left(vertices[i-1], vertices[i], vertices[i+1]):
                    is_convex = False
                    
    # We also want to append the last point :)
    p_new.append(v_last)
    
    # Returns: (1) New Pointlist, (2) Soft reduced pointlist (reduce_poly())
    return p_new, is_convex

    """ OLD FUNCTION: """    
    # Wipe all points too close to each other
    vxx = vertices = reduce_poly(vertices, minlen)
    
    # Create Output List
    p_new = []
    p_new.append(vertices[0])
    
    # Set the starting vertice
    p_old = vertices[0]
    alpha_old = None

    # For each vector, compare the angle difference to the last one
    for i in range(1, len(vertices)):
        x1, y1 = p_old
        x2, y2 = vertices[i]
        p_old = (x2, y2)

        # Make Vector
        vx, vy = (x2-x1, y2-y1)
    
        # Vector length
        l = sqrt((vx*vx) + (vy*vy))
        
        # normalize
        vx /= l
        vy /= l
        
        # Get Angle
        if fabs(vx) < 0.2:
            alpha = 90
        else: 
            alpha = degrees(atan2(vy,vx))

        if alpha_old == None:
            alpha_old = alpha
            continue
        
        # Get difference to previous angle
        alpha_diff = fabs(alpha - alpha_old)        
        alpha_old = alpha
        
        # If the new vector differs from the old one, we add the old point
        # to the output list, as the line changed it's way :)
        if alpha_diff > tolerance:
            #print ">",alpha_diff, "\t", vx, vy, l
            p_new.append(vertices[i-1])

    # We also want to append the last point :)
    p_new.append(vertices[-1])
    
    # Returns: (1) New Pointlist, (2) Soft reduced pointlist (reduce_poly())
    return p_new, vxx
    
        
# The following functions is_left, reduce_poly and convex_hull are 
# from the pymunk project (http://code.google.com/p/pymunk/)
def is_left(p0, p1, p2):
    """Test if p2 is left, on or right of the (infinite) line (p0,p1).
    
    :return: > 0 for p2 left of the line through p0 and p1
        = 0 for p2 on the line
        < 0 for p2 right of the line
    """
    sorting = (p1[0] - p0[0])*(p2[1]-p0[1]) - (p2[0]-p0[0])*(p1[1]-p0[1])
    if sorting > 0: return 1
    elif sorting < 0: return -1 
    else: return 0

def is_convex(points):
    """Test if a polygon (list of (x,y)) is strictly convex or not.
    
    :return: True if the polygon is convex, False otherwise
    """
    #assert len(points) > 2, "not enough points to form a polygon"
    
    p0 = points[0]
    p1 = points[1]
    p2 = points[2]

    xc, yc = 0, 0
    is_same_winding = is_left(p0, p1, p2)
    for p2 in points[2:] + [p0] + [p1]:
        if is_same_winding != is_left(p0, p1, p2): 
            return False
        a = p1[0] - p0[0], p1[1] - p0[1] # p1-p0
        b = p2[0] - p1[0], p2[1] - p1[1] # p2-p1
        if sign(a[0]) != sign(b[0]): xc +=1
        if sign(a[1]) != sign(b[1]): yc +=1
        p0, p1 = p1, p2
   
    return xc <= 2 and yc <= 2

def sign(x): 
    if x < 0: return -1 
    else: return 1


def reduce_poly(points, tolerance=50):
    """Remove close points to simplify a polyline
    tolerance is the min distance between two points squared.
    
    :return: The reduced polygon as a list of (x,y)
    """
    curr_p = points[0]
    reduced_ps = [points[0]]
    
    for p in points[1:]:
        x1, y1 = curr_p
        x2, y2 = p
        dx = fabs(x2 - x1)
        dy = fabs(y2 - y1)
        l = sqrt((dx*dx) + (dy*dy))
        if l > tolerance:
            curr_p = p
            reduced_ps.append(p)
            
    return reduced_ps

def convex_hull(points):
    """Create a convex hull from a list of points.
    This function uses the Graham Scan Algorithm.
    
    :return: Convex hull as a list of (x,y)
    """
    ### Find lowest rightmost point
    p0 = points[0]
    for p in points[1:]:
        if p[1] < p0[1]:
            p0 = p
        elif p[1] == p0[1] and p[0] > p0[0]:
            p0 = p
    points.remove(p0)
    
    ### Sort the points angularly about p0 as center
    f = partial(is_left, p0)
    points.sort(cmp = f)
    points.reverse()
    points.insert(0, p0)
    
    ### Find the hull points
    hull = [p0, points[1]]
    
    for p in points[2:]:
        
        pt1 = hull[-1]
        pt2 = hull[-2]
        l = is_left(pt2, pt1, p) 
        if l > 0:
            hull.append(p)
        else:
            while l <= 0 and len(hull) > 2:
                hull.pop()
                pt1 = hull[-1]
                pt2 = hull[-2]
                l = is_left(pt2, pt1, p)
            hull.append(p)
    return hull     
    



# -*- encoding: utf-8 -*-
# Pilas engine - A video game framework.
#
# Copyright 2010 - Hugo Ruscitti
# License: LGPLv3 (see http://www.gnu.org/licenses/lgpl.html)
#
# Website - http://www.pilas-engine.com.ar

from pilas.actores import Actor
import pilas

class Mono(Actor):
    """Representa la cara de un mono de color marrón.

    Este personaje se usa como ejemplo básico de un actor. Sus
    métodos mas usados son:

    - sonreir
    - gritar

    El primero hace que el mono se ría y el segundo que grite.
    """

    def __init__(self, x=0, y=0):
        # carga las imagenes adicionales.
        self.image_normal = pilas.imagenes.cargar('monkey_normal.png')
        self.image_smile = pilas.imagenes.cargar('monkey_smile.png')
        self.image_shout = pilas.imagenes.cargar('monkey_shout.png')

        self.sound_shout = pilas.sonidos.cargar('shout.wav')
        self.sound_smile = pilas.sonidos.cargar('smile.wav')

        # Inicializa el actor.
        Actor.__init__(self, self.image_normal, x=x, y=y)
        self.radio_de_colision = 50

    def sonreir(self):
        self.definir_imagen(self.image_smile)
        # Luego de un segundo regresa a la normalidad
        pilas.mundo.agregar_tarea(0.5, self.normal)
        self.sound_smile.reproducir()

    def gritar(self):
        self.definir_imagen(self.image_shout)
        # Luego de un segundo regresa a la normalidad
        pilas.mundo.agregar_tarea(1, self.normal)
        self.sound_shout.reproducir()

    def normal(self):
        self.definir_imagen(self.image_normal)



# -*- encoding: utf-8 -*-
# Pilas engine - A video game framework.
#
# Copyright 2010 - Hugo Ruscitti
# License: LGPLv3 (see http://www.gnu.org/licenses/lgpl.html)
#
# Website - http://www.pilas-engine.com.ar

import dispatch

mueve_mouse = dispatch.Signal(providing_args=['x', 'y', 'dx', 'dy'])
click_de_mouse = dispatch.Signal(providing_args=['button', 'x', 'y'])
termina_click = dispatch.Signal(providing_args=['button', 'x', 'y'])
mueve_rueda = dispatch.Signal(providing_args=['delta'])
pulsa_tecla = dispatch.Signal(providing_args=['code'])
pulsa_tecla_escape = dispatch.Signal(providing_args=[])
actualizar = dispatch.Signal(providing_args=[])

# Se emite cuando el mundo ingresa o sale del modo depuracion (pulsando F12)
inicia_modo_depuracion = dispatch.Signal(providing_args=[]) 
sale_modo_depuracion = dispatch.Signal(providing_args=[])
actualiza_modo_depuracion = dispatch.Signal(providing_args=[])



# -*- encoding: utf-8 -*-
# pilas engine - a video game framework.
#
# copyright 2010 - hugo ruscitti
# license: lgplv3 (see http://www.gnu.org/licenses/lgpl.html)
#
# website - http://www.pilas-engine.com.ar

'''
Pilas
=====

Pilas es un motor para crear videojuegos de manera
simple y didáctica.

Para iniciar el módulo debes llamar a la función ``iniciar``::

    pilas.iniciar()

y opcionalmente puedes indicarle parámetros si quieres. Te
recomiendo ejecutar la siguiente funcion para obtener
mas informacion::

    help(pilas.iniciar)

Ten en cuenta que este motor se pude ejecutar directamente
desde la consola interactiva de python, así podrás investigar
y divertirte de forma interactiva.

Puedes obtener mas información en nuestro sitio:

    - http:://www.pilas-engine.com.ar

'''


mundo = None
motor = None
bg = None
__all__ = ['actores', 'iniciar', 'terminar', 'ejecutar', 'interpolar', 'avisar']

import utils
import motores
import elements

# Carga el modulo de autocompletado si esta
# en sesion interactiva.
if utils.esta_en_sesion_interactiva():
    utils.cargar_autocompletado()

def iniciar(ancho=640, alto=480, titulo='Pilas', usar_motor='pysfml', modo='detectar'):
    """Inicia el motor y abre la ventana principal del videojuego.
    
    Esta funcion se ejecuta al principio de cualquier juego, porque
    además de crear la ventana principal inicializa otros submódulos
    de pilas que te permites hacer mas cosas.
    
    Un ejemplo de invocación para esta función es::

        pilas.iniciar()

    aunque también puedes indicarle puntualmente el valor de algún
    argumento:

        pilas.iniciar(titulo='titulo de mi juego')

    """
    global mundo
    global motor


    (motores_disponibles, recomendado) = obtener_motores_disponibles()

    if not recomendado:
        raise Exception("No se encuentran bibliotecas multimedia, tiene que instalar SFML o pygame.")

    if usar_motor not in motores_disponibles.keys():
        print "Advertencia: El sistema solicitado '%s' no esta disponible, cargado '%s' en su lugar." %(usar_motor, recomendado)
        usar_motor = recomendado

    if usar_motor not in ['sfml', 'pysfml']:
        print "Advertencia: Te recomendamos usar SFML, busca en la web de pilas como instalarlo..."

    motor = motores_disponibles[usar_motor]()

    import simbolos
    import os
    import sys
    import time

    import baseactor
    import colores
    import imagenes
    import sonidos
    import actores
    import fps
    from interpolaciones import Lineal
    import dispatch
    import eventos
    import habilidades
    import ventana
    import comportamientos
    import escenas
    import fondos
    from control import Control
    from camara import Camara
    import copy
    import pilas.utils
    from mundo import Mundo
    from colisiones import Colisiones
    import random
    import ejemplos
    import red
    import motores
    import atajos
    import fisica
    import grupo

    pilas.colisiones = Colisiones()

    # Cuando inicia en modo interactivo se asegura
    # de crear la ventana dentro del mismo hilo que
    # tiene el contexto opengl.
    if modo == 'detectar':
        if utils.esta_en_sesion_interactiva():
            iniciar_y_cargar_en_segundo_plano(ancho, alto, titulo + " [Modo Interactivo]")
        else:
            mundo = pilas.mundo.Mundo(ancho, alto, titulo)
            escenas.Normal()
    elif modo == 'interactivo':
        iniciar_y_cargar_en_segundo_plano(ancho, alto, titulo + " [Modo Interactivo]")
    else:
        raise Exception("Lo siento, el modo indicado es invalido, solo se admite 'interactivo' y 'detectar'")


def iniciar_y_cargar_en_segundo_plano(ancho, alto, titulo):
    "Ejecuta el bucle de pilas en segundo plano."
    import threading
    global gb

    bg = threading.Thread(target=iniciar_y_ejecutar, args=(ancho, alto, titulo))
    bg.start()

def reiniciar():
    actores.utils.eliminar_a_todos()

def iniciar_y_ejecutar(ancho, alto, titulo, ignorar_errores=False):
    from mundo import Mundo
    global mundo

    mundo = Mundo(ancho, alto, titulo)
    escenas.Normal()
    ejecutar(ignorar_errores)

def terminar():
    "Finaliza la ejecución de pilas y cierra la ventana principal."
    global mundo

    if mundo:
        mundo.terminar()
    else:
        print "No se puede terminar pilas porque no la has inicializado."

def ejecutar(ignorar_errores=False):
    """Pone en funcionamiento el ejecutar principal.

    Esta función se comporta diferente en modo interactivo y modo script:

        - En modo interactivo pilas generará un hilo que te permitirá seguir
          escribiendo comandos interactivamente y viendo los resultados en la
          ventana.

        - En modo script, la función bloqueará la ejecución lineal de tu
          script y llevará todo el control al bucle de juego interno de
          pilas. Por lo que es buena idea poner la linea ``pilas.ejecutar()`` al
          final del script...
    """
    global mundo

    if mundo:
        mundo.ejecutar_bucle_principal(ignorar_errores)
    else:
        raise Exception("Tienes que llamar a pilas.iniciar() antes de ejecutar el juego.")


def interpolar(valor_o_valores, duracion=1, demora=0, tipo='lineal'):
    """Retorna un objeto que representa cambios de atributos progresivos.
    
    El resultado de esta función se puede aplicar a varios atributos
    de los actores, por ejemplo::
        
        bomba = pilas.actores.Bomba()
        bomba.escala = pilas.interpolar(3)

    Esta función también admite otros parámetros cómo:

        - duracion: es la cantidad de segundos que demorará toda la interpolación.
        - demora: cuantos segundos se deben esperar antes de iniciar.
        - tipo: es el algoritmo de la interpolación, puede ser 'lineal'.
    """


    import interpolaciones

    algoritmos = {
            'lineal': interpolaciones.Lineal,
            }

    if algoritmos.has_key('lineal'):
        clase = algoritmos[tipo]
    else:
        raise ValueError("El tipo de interpolacion %s es invalido" %(tipo))

    # Permite que los valores de interpolacion sean un numero o una lista.
    if not isinstance(valor_o_valores, list):
        valor_o_valores = [valor_o_valores]

    return clase(valor_o_valores, duracion, demora)

anterior_texto = None

def avisar(mensaje):
    "Emite un mensaje en la ventana principal."
    global anterior_texto

    if anterior_texto:
        anterior_texto.eliminar()

    texto = actores.Texto(mensaje)
    texto.magnitud = 22
    texto.izquierda = -310
    texto.abajo = -230
    anterior_texto = texto

def ver(objeto):
    "Imprime en pantalla el codigo fuente asociado a un objeto o elemento de pilas."
    import inspect

    try:
        codigo = inspect.getsource(objeto.__class__)
    except TypeError:
        codigo = inspect.getsource(objeto)

    print codigo

def obtener_motores_disponibles():
    """Retorna una tupla con dos valores: los motores disponibles y el recomendado.

    Dado que pilas soporta dos motores multimadia (SFML y Pygame), esta
    función intentará decetar cual de estos sistemas multimedia esta
    disponible en el sistema.
    """

    motores_disponibles = {}
    sistema_recomendado = None
    import pilas.motores

    try:
        motores_disponibles['pygame'] = pilas.motores.Pygame
        sistema_recomendado = "pygame"
    except AttributeError:
        pass

    try:
        motores_disponibles['sfml'] = pilas.motores.pySFML
        motores_disponibles['pysfml'] = pilas.motores.pySFML
        sistema_recomendado = "sfml"
    except AttributeError:
        pass

    return (motores_disponibles, sistema_recomendado)



"""
This file is part of the 'Elements' Project
Elements is a 2D Physics API for Python (supporting Box2D2)

Copyright (C) 2008, The Elements Team, <elements@linuxuser.at>

Home:  http://elements.linuxuser.at
IRC:   #elements on irc.freenode.org

Code:  http://www.assembla.com/wiki/show/elements
       svn co http://svn2.assembla.com/svn/elements                     

License:  GPLv3 | See LICENSE for the full text
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.              
"""
from locals import *
from elements import box2d

# Imports
from math import pi
from math import sqrt
from math import asin

import tools_poly

class Add:
    element_count = 0
    
    def __init__(self, parent):
        self.parent = parent
        
    def ground(self):
        """ Add a static ground to the scene
        
            Return: box2d.b2Body
        """
        return self._rect((-10.0, 0.0), 50.0, 0.1, dynamic=False)
            
    def triangle(self, pos, sidelength, dynamic=True, density=1.0, restitution=0.16, friction=0.5, screenCoord=True):
        """ Add a triangle | pos & a in the current input unit system (meters or pixels)
        
            Parameters:
              pos .... position (x,y)
              sidelength ...... sidelength
              other .. see [physics parameters]
            
            Return: box2d.b2Body
        """
        vertices = [(-sidelength, 0.0), (sidelength, 0.0), (0.0, 2*sidelength)]
        return self.poly(pos, vertices, dynamic, density, restitution, friction, screenCoord)

    def ball(self, pos, radius, dynamic=True, density=1.0, restitution=0.16, friction=0.5, screenCoord=True):
        """ Add a dynamic ball at pos after correcting the positions and legths to the internal
            meter system if neccessary (if INPUT_PIXELS), then call self._add_ball(...)
            
            Parameters:
              pos ..... position (x,y)
              radius .. circle radius
              other ... see [physics parameters]
              
            Return: box2d.b2Body
        """
        # Bring coordinates into the world coordinate system (flip, camera offset, ...)
        if screenCoord: x, y = self.parent.to_world(pos)
        else: x, y = pos


        if self.parent.input == INPUT_PIXELS:
            x /= self.parent.ppm
            y /= self.parent.ppm
            radius /= self.parent.ppm
            
        return self._ball((x,y), radius, dynamic, density, restitution, friction)

    def _ball(self, pos, radius, dynamic=True, density=1.0, restitution=0.16, friction=0.5):
        # Add a ball without correcting any settings
        # meaning, pos and vertices are in meters
        # Define the body
        x, y = pos
        bodyDef = box2d.b2BodyDef()
        bodyDef.position=(x, y)

        userData = { 'color' : self.parent.get_color() }
        bodyDef.userData = userData

        # Create the Body
        if not dynamic:
            density = 0

        body = self.parent.world.CreateBody(bodyDef)
                    
        self.parent.element_count += 1

        # Add a shape to the Body
        circleDef = box2d.b2CircleDef()
        circleDef.density = density
        circleDef.radius = radius
        circleDef.restitution = restitution
        circleDef.friction = friction

        body.CreateShape(circleDef)
        body.SetMassFromShapes()    
        
        return body    

    def rect(self, pos, width, height, angle=0, dynamic=True, density=1.0, restitution=0.16, friction=0.5, screenCoord=True):
        """ Add a dynamic rectangle with input unit according to self.input (INPUT_PIXELS or INPUT_METERS)
            Correcting the positions to meters and calling self._add_rect()

            Parameters:
              pos ..... position (x,y)
              width ....... horizontal line
              height ....... vertical line
              angle ........ in degrees (0 .. 360)
              other ... see [physics parameters]

            Return: box2d.b2Body
        """
        # Bring coordinates into the world coordinate system (flip, camera offset, ...)
        if screenCoord: x, y = self.parent.to_world(pos)
        else: x, y = pos

        # If required, translate pixel -> meters
        if self.parent.input == INPUT_PIXELS:
            x /= self.parent.ppm
            y /= self.parent.ppm
            width /= self.parent.ppm
            height /= self.parent.ppm
        
        # grad -> radians
        angle = (angle * pi) / 180
        
        return self._rect((x,y), width, height, angle, dynamic, density, restitution, friction)


    def wall(self, pos1, pos2, width=5, density=1.0, restitution=0.16, friction=0.5, screenCoord=True):
        """ Add a static rectangle between two arbitrary points with input unit according to self.input 
            (INPUT_PIXELS or INPUT_METERS) Correcting the positions to meters and calling self._add_rect()

            Return: box2d.b2Body
        """
        if width < 5: width = 5
        
        if (pos1[0] < pos2[0]):        
            x1, y1 = pos1
            x2, y2 = pos2
        else:
            x1, y1 = pos2
            x2, y2 = pos1          

        # Bring coordinates into the world coordinate system (flip, camera offset, ...)
        if screenCoord: 
           x1, y1 = self.parent.to_world((x1, y1))
           x2, y2 = self.parent.to_world((x2, y2))

        # If required, translate pixel -> meters
        if self.parent.input == INPUT_PIXELS:
            x1 /= self.parent.ppm
            y1 /= self.parent.ppm
            x2 /= self.parent.ppm
            y2 /= self.parent.ppm
            width /= self.parent.ppm
                           
        length = sqrt( (x1-x2)*(x1-x2) + (y1-y2)*(y1-y2) )*0.5

        if width > 0:
            halfX = x1 + (x2-x1)*0.5
            halfY = y1 + (y2-y1)*0.5

            angle = asin( (y2-halfY)/length )
            return self._rect((halfX, halfY), length, width, angle, False, density, restitution, friction)
                    
    def _rect(self, pos, width, height, angle=0, dynamic=True, density=1.0, restitution=0.16, friction=0.5):
        # Add a rect without correcting any settings
        # meaning, pos and vertices are in meters
        # angle is now in radians ((degrees * pi) / 180))
        x, y = pos
        bodyDef = box2d.b2BodyDef()
        bodyDef.position=(x, y)

        userData = { 'color' : self.parent.get_color() }
        bodyDef.userData = userData

        # Create the Body
        if not dynamic:
            density = 0

        body = self.parent.world.CreateBody(bodyDef)
                    
        self.parent.element_count += 1

        # Add a shape to the Body
        boxDef = box2d.b2PolygonDef()
        
        boxDef.SetAsBox(width, height, (0,0), angle)
        boxDef.density = density
        boxDef.restitution = restitution
        boxDef.friction = friction
        body.CreateShape(boxDef)
        
        body.SetMassFromShapes()
        
        return body

    def poly(self, pos, vertices, dynamic=True, density=1.0, restitution=0.16, friction=0.5, screenCoord=True):
        """ Add a dynamic polygon, which has the vertices arranged around the poly's center at pos
            Correcting the positions to meters if INPUT_PIXELS, and calling self._add_poly()

            Parameters:
              pos ....... position (x,y)
              vertices .. vertices arranged around the center
              other ... see [physics parameters]

            Return: box2d.b2Body
        """        
        # Bring coordinates into the world coordinate system (flip, camera offset, ...)
        if screenCoord: x, y = self.parent.to_world(pos)
        else: x, y = pos
                    
        # If required, translate pixel -> meters
        if self.parent.input == INPUT_PIXELS:
            # translate pixel -> meters
            x /= self.parent.ppm
            y /= self.parent.ppm
            
            # Translate vertices from pixels to meters
            v_new = []
            for v in vertices:
                vx, vy = v
                v_new.append((vx/self.parent.ppm, vy/self.parent.ppm))
            vertices = v_new
        
        return self._poly((x,y), vertices, dynamic, density, restitution, friction)

    def _poly(self, pos, vertices, dynamic=True, density=1.0, restitution=0.16, friction=0.5):
        # add a centered poly at pos without correcting any settings
        # meaning, pos and vertices are in meters
        x, y = pos
        bodyDef = box2d.b2BodyDef()
        bodyDef.position=(x, y)
            
        userData = { 'color' : self.parent.get_color() }
        bodyDef.userData = userData

        # Create the Body
        if not dynamic:
            density = 0

        body = self.parent.world.CreateBody(bodyDef)
        
        self.parent.element_count += 1

        # Add a shape to the Body
        polyDef = box2d.b2PolygonDef()

        polyDef.setVertices(vertices)
        polyDef.density = density
        polyDef.restitution = restitution
        polyDef.friction = friction

        body.CreateShape(polyDef)
        body.SetMassFromShapes()
                
        return body
   
    def concavePoly(self, vertices, dynamic=True, density=1.0, restitution=0.16, friction=0.5, screenCoord=True):
        # 1. Step: Reduce        
        # Detect if the polygon is closed or open
        if vertices[0] != vertices[-1]:        
            is_closed = False
        else:
            is_closed = True
                            
        # Continue reducing the vertecs
        x, y = c = tools_poly.calc_center(vertices)
        vertices = tools_poly.poly_center_vertices(vertices)
        
        # Bring coordinates into the world coordinate system (flip, camera offset, ...)
        if screenCoord: x, y = self.parent.to_world(c)
        else: x, y = c

        # If required, translate pixel -> meters
        if self.parent.input == INPUT_PIXELS:
            # translate pixel -> meters
            x /= self.parent.ppm
            y /= self.parent.ppm
        
        # Let's add the body
        bodyDef = box2d.b2BodyDef()
        bodyDef.position=(x, y)

        userData = { 'color' : self.parent.get_color() }
        bodyDef.userData = userData

        # Create the Body
        if not dynamic:
            density = 0

        body = self.parent.world.CreateBody(bodyDef)
                    
        self.parent.element_count += 1

        # Create the reusable Box2D polygon and circle definitions
        polyDef = box2d.b2PolygonDef()
        polyDef.vertexCount = 4 # rectangle
        polyDef.density = density
        polyDef.restitution = restitution
        polyDef.friction = friction

        circleDef = box2d.b2CircleDef()
        circleDef.density = density
        circleDef.radius = 0.086
        circleDef.restitution = restitution
        circleDef.friction = friction

        # Set the scale factor 
        factor = 8.0

        v2 = box2d.b2Vec2(*vertices[0])
        for v in vertices[1:]:
            v1    = v2.copy()
            v2    = box2d.b2Vec2(*v)
             
            vdir = v2-v1 # (v2x-v1x, v2y-v1y)
            vdir.Normalize()
            
            # we need a little size for the end part
            vn = box2d.b2Vec2(-vdir.y*factor, vdir.x*factor)

            v = [ v1+vn, v1-vn, v2-vn, v2+vn ] 

            # Create a line (rect) for each part of the polygon, 
            # and attach it to the body                
            polyDef.setVertices( [vi / self.parent.ppm for vi in v] )

            try:
                polyDef.checkValues()
            except ValueError:
                print "concavePoly: Created an invalid polygon!"
                return None

            body.CreateShape(polyDef)

            # Now add a circle to the points between the rects
            # to avoid sharp edges and gaps
            if not is_closed and v2.tuple() == vertices[-1]:
                # Don't add a circle at the end
                break

            circleDef.localPosition = v2 / self.parent.ppm
            body.CreateShape(circleDef)            
                                 
        # Now, all shapes have been attached
        body.SetMassFromShapes()                
        
        # Return hard and soft reduced vertices
        return body

    def complexPoly(self, vertices, dynamic=True, density=1.0, restitution=0.16, friction=0.5):
        # 1. Step: Reduce
        # 2. Step: See if start and end are close, if so then close the polygon
        # 3. Step: Detect if convex or concave
        # 4. Step: Start self.convexPoly or self.concavePoly
        vertices, is_convex = tools_poly.reduce_poly_by_angle(vertices)            
        #print "->", is_convex

        # If start and endpoints are close to each other, close polygon
        x1, y1 = vertices[0]
        x2, y2 = vertices[-1]
        dx = x2 - x1
        dy = y2 - y1
        l = sqrt((dx*dx)+(dy*dy))

        if l < 50:
            vertices[-1] = vertices[0]
        else:
            # Never convex if open (we decide so :)
            is_convex = False

        if tools_poly.is_line(vertices):
            # Lines shall be drawn by self.concavePoly(...)
            print "is line"
            is_convex = False
                    
        if is_convex:
            print "convex"
            return self.convexPoly(vertices, dynamic, density, restitution, friction), vertices
        else:
            print "concave"
            return self.concavePoly(vertices, dynamic, density, restitution, friction), vertices        
        

    def convexPoly(self, vertices, dynamic=True, density=1.0, restitution=0.16, friction=0.5):
        """ Add a complex polygon with vertices in absolute positions (meters or pixels, according
            to INPUT_PIXELS or INPUT_METERS). This function does the reduction and convec hulling 
            of the poly, and calls add_poly(...) 

            Parameters:
              vertices .. absolute vertices positions
              other ..... see [physics parameters]

            Return: box2d.b2Body
        """
        # NOTE: Box2D has a maximum poly vertex count, defined in Common/box2d.b2Settings.h (box2d.b2_maxPolygonVertices)
        # We need to make sure, that we reach that by reducing the poly with increased tolerance
        # Reduce Polygon
        tolerance = 10 #5
        v_new = vertices
        while len(v_new) > box2d.b2_maxPolygonVertices:
            tolerance += 1    
            v_new = tools_poly.reduce_poly(vertices, tolerance)
            
        print "convexPoly: Polygon reduced from %i to %i vertices | tolerance: %i" % (len(vertices), len(v_new), tolerance)
        vertices = v_new
             
        # So poly should be alright now
        # Continue reducing the vertecs
        vertices_orig_reduced = vertices    
        vertices = tools_poly.poly_center_vertices(vertices)

        vertices = tools_poly.convex_hull(vertices)

        if len(vertices) < 3: 
            return 
        
        # Define the body
        x, y = c = tools_poly.calc_center(vertices_orig_reduced)
        return self.poly((x,y), vertices, dynamic, density, restitution, friction)

    def to_b2vec(self, pt):
    # Convert vector to a b2vect
        pt = self.parent.to_world(pt)
        ptx, pty = pt
        ptx /= self.parent.ppm
        pty /= self.parent.ppm
        pt = box2d.b2Vec2(ptx, pty)
        return pt

    def joint(self, *args):
        print "* Add Joint:", args

        if len(args) == 4:
            # Distance Joint
            b1, b2, p1, p2 = args

            p1 = self.to_b2vec(p1)
            p2 = self.to_b2vec(p2)            
           
            jointDef = box2d.b2DistanceJointDef()
            jointDef.Initialize(b1, b2, p1, p2)
            jointDef.collideConnected = True
            
            self.parent.world.CreateJoint(jointDef)           
             
        elif len(args) == 3:
            # Revolute Joint between two bodies (unimplemented)
            pass

        elif len(args) == 2:
            # Revolute Joint to the Background, at point 
            b1 = self.parent.world.GetGroundBody()
            b2 = args[0]
            p1 = self.to_b2vec(args[1])

            jointDef = box2d.b2RevoluteJointDef()
            jointDef.Initialize(b1, b2, p1)
            self.parent.world.CreateJoint(jointDef)

        elif len(args) == 1:
            # Revolute Joint to the Background, body center
            b1 = self.parent.world.GetGroundBody()
            b2 = args[0]
            p1 = b2.GetWorldCenter()
            
            jointDef = box2d.b2RevoluteJointDef()
            jointDef.Initialize(b1, b2, p1)
            
            self.parent.world.CreateJoint(jointDef)

    def motor(self, body, pt, torque=900, speed=-10):
        # Revolute joint to the background with motor torque applied
        b1 = self.parent.world.GetGroundBody()
        pt = self.to_b2vec(pt)

        jointDef = box2d.b2RevoluteJointDef()
        jointDef.Initialize(b1, body, pt)
        jointDef.maxMotorTorque = torque
        jointDef.motorSpeed = speed
        jointDef.enableMotor = True

        self.parent.world.CreateJoint(jointDef)

    def mouseJoint(self, body, pos, jointForce=100.0):
        pos = self.parent.to_world(pos)
        x, y = pos
        x /= self.parent.ppm
        y /= self.parent.ppm

        mj = box2d.b2MouseJointDef()
        mj.body1 = self.parent.world.GetGroundBody()
        mj.body2 = body
        mj.target = (x, y)
        mj.maxForce = jointForce * body.GetMass()
        self.parent.mouseJoint = self.parent.world.CreateJoint(mj).getAsType()
        body.WakeUp()
        
    def remove_mouseJoint(self):
        if self.parent.mouseJoint:
            self.parent.world.DestroyJoint(self.parent.mouseJoint)
            self.parent.mouseJoint = None




# -*- encoding: utf-8 -*-
# pilas engine - a video game framework.
#
# copyright 2010 - hugo ruscitti
# license: lgplv3 (see http://www.gnu.org/licenses/lgpl.html)
#
# website - http://www.pilas-engine.com.ar

import SocketServer



def iniciar_servidor():


    class EchoRequestHandler(SocketServer.BaseRequestHandler ):
        def setup(self):
            print self.client_address, 'connected!'
            self.request.send('hi ' + str(self.client_address) + '\n')

        def handle(self):
            data = 'dummy'
            while data:
                data = self.request.recv(1024)
                print "ha llegado el mensaje:", data
                self.request.send(data)
                
                if data.strip() == 'bye':
                    return

        def finish(self):
            print self.client_address, 'disconnected!'
            self.request.send('bye ' + str(self.client_address) + '\n')


    #server host is a tuple ('host', port)
    puerto = 50008
    print "iniciando el modo servidor en el puerto %d" %(puerto)

    servidor = SocketServer.ThreadingTCPServer(('', puerto), EchoRequestHandler)
    servidor.serve_forever()



# -*- encoding: utf-8 -*-
import pilas

pilas.iniciar()
mono = pilas.actores.Mono()
mono.aprender(pilas.habilidades.SeguirClicks)
mono.aprender(pilas.habilidades.AumentarConRueda)

pilas.avisar("El mono sigue los clicks, y cambia de tamaño si mueves la\nrueda del mouse.")
pilas.ejecutar()



# -*- encoding: utf-8 -*-
# pilas engine - a video game framework.
#
# copyright 2010 - hugo ruscitti
# license: lgplv3 (see http://www.gnu.org/licenses/lgpl.html)
#
# website - http://www.pilas-engine.com.ar

import pilas

def comer(mono, banana):
    mono.sonreir()
    banana.eliminar()

class Colisiones(pilas.escenas.Normal):
    """Es una escena que tiene un mono y algunas frutas para comer."""


    def __init__(self):
        pilas.escenas.Normal.__init__(self, pilas.colores.grisoscuro)
        self.crear_personajes()

        pilas.colisiones.agregar([self.mono], self.bananas, comer)
        pilas.avisar("Utilice el mouse para mover al mono y comer")

    def crear_personajes(self):
        self.mono = pilas.actores.Mono()
        self.mono.aprender(pilas.habilidades.Arrastrable)

        self.bananas = pilas.atajos.fabricar(pilas.actores.Banana, 20)
        






# -*- encoding: utf-8 -*-
# pilas engine - a video game framework.
#
# copyright 2010 - hugo ruscitti
# license: lgplv3 (see http://www.gnu.org/licenses/lgpl.html)
#
# website - http://www.pilas-engine.com.ar

import utils
import pilas

class Colisiones:
    "Administra todas las colisiones entre actores."

    def __init__(self):
        self.colisiones = []

    def verificar_colisiones(self):

        for x in self.colisiones:
            self._verificar_colisiones_en_tupla(x)

    def _verificar_colisiones_en_tupla(self, tupla):
        "Toma dos grupos de actores y analiza colisiones entre ellos."
        (grupo_a, grupo_b, funcion_a_llamar) = tupla

        for a in grupo_a:
            for b in grupo_b:
                if id(a) != id(b) and utils.colisionan(a, b):
                    funcion_a_llamar(a, b)

                    # verifica si alguno de los dos objetos muere en la colision.
                    if a not in pilas.actores.todos:
                        if a in grupo_a:
                            list.remove(grupo_a, a)

                    if b not in pilas.actores.todos:
                        if b in grupo_b:
                            list.remove(grupo_b, b)



    def agregar(self, grupo_a, grupo_b, funcion_a_llamar):
        "Agrega dos listas de actores para analizar colisiones."

        if not isinstance(grupo_a, list):
            grupo_a = [grupo_a]

        if not isinstance(grupo_b, list):
            grupo_b = [grupo_b]

        self.colisiones.append((grupo_a, grupo_b, funcion_a_llamar))

    def obtener_colisiones(self, actor, grupo_de_actores):
        "Retorna una lista de los actores que colisionan con uno en particular."

        lista_de_colisiones = []

        for a in grupo_de_actores:
            if id(actor) != id(a) and utils.colisionan(actor, a):
                lista_de_colisiones.append(a)

        return lista_de_colisiones




# -*- encoding: utf-8 -*-
# Pilas engine - A video game framework.
#
# Copyright 2010 - Hugo Ruscitti
# License: LGPLv3 (see http://www.gnu.org/licenses/lgpl.html)
#
# Website - http://www.pilas-engine.com.ar

import pilas


class Camara(object):
    """Representa el punto de vista de la ventana.

    Los atributos ``x`` e ``y`` indican cual debe ser el
    punto central de la pantalla. Por defecto estos
    valores con (0, 0)."""

    def __init__(self, app):
        pass

    def _set_x(self, x):
        if pilas.utils.es_interpolacion(x):
            x.apply(self, function='_set_x')
        else:
            pilas.motor.definir_centro_de_la_camara(x, self.y)

    def _get_x(self):
        x, y = pilas.motor.obtener_centro_de_la_camara()
        return x

    def _set_y(self, y):
        if pilas.utils.es_interpolacion(y):
            y.apply(self, function='_set_y')
        else:
            pilas.motor.definir_centro_de_la_camara(self.x, -y)

    def _get_y(self):
        x, y = pilas.motor.obtener_centro_de_la_camara()
        return y

    x = property(_get_x, _set_x)
    y = property(_get_y, _set_y)




import pilas

class EscenaMenu(pilas.escenas.Escena):
    "Escena de presentacion del juego."

    def __init__(self):
        pilas.actores.utils.eliminar_a_todos()
        pilas.fondos.Fondo('data/inicio.png')
        self.menu_de_juego()

    def menu_de_juego(self):
        opciones_menu = [
        ("Iniciar juego", self.iniciar_juego),
        ("Ayuda", self.mostrar_ayuda),
        ("Salir", self.salir_del_juego),
        ]
        self.menu = pilas.actores.Menu(opciones_menu, y = -50)

    def iniciar_juego(self):
        import escena_juego
        escena_juego.Juego()

    def salir_del_juego(self):
        pilas.terminar()

    def mostrar_ayuda(self):
        import escena_ayuda
        escena_ayuda.Ayuda()



# -*- encoding: utf-8 -*-
# Pilas engine - A video game framework.
#
# Copyright 2010 - Hugo Ruscitti
# License: LGPLv3 (see http://www.gnu.org/licenses/lgpl.html)
#
# Website - http://www.pilas-engine.com.ar

import array
import cairo

import pilas
from pilas.actores import Actor

class Lapiz(object):

    def __init__(self):
        self.x = 0
        self.y = 0

    def set_x(self, x):
        self._x = x + 320

    def get_x(self):
        return self._x

    def set_y(self, y):
        self._y = 240 - y 

    def get_y(self):
        return self._y

    x = property(get_x, set_x)
    y = property(get_y, set_y)
    

class Pizarra(Actor):
    """Representa una superficie de dibujo inicialmente transparente.

    Puedes pintar sobre esta pizarra usando métodos que simulan
    un lapiz, que se puede mover sobre una superficie.
    """

    def __init__(self, x=0, y=0):
        Actor.__init__(self, x=x, y=y)
        self.canvas = pilas.motor.Canvas()
        self.lapiz = Lapiz()
        self.actualizar_imagen()
        self.levantar_lapiz()
        self.mover_lapiz(0, 0)
        self.definir_centro(320, 240)
        self.habilitar_actualizacion_automatica()

    def habilitar_actualizacion_automatica(self):
        self.actualiza_automaticamente = True
        self.actualizar_imagen()

    def deshabilitar_actualizacion_automatica(self):
        self.actualiza_automaticamente = False

    def levantar_lapiz(self):
        self.lapiz_bajo = False

    def bajar_lapiz(self):
        self.lapiz_bajo = True

    def actualizar_imagen(self):
        "Se encarga de actualizar la vista de la pizarra."
        self.canvas.actualizar()
        self.definir_imagen(self.canvas.image)

    def pintar_punto(self, x, y):
        y = 240 - y
        x += 320
        self.canvas.context.arc(x, y, 10, 0, 2*3.1415)
        self.canvas.context.fill()
        if self.actualiza_automaticamente:
            self.actualizar_imagen()

    def mover_lapiz(self, x, y):
        if self.lapiz_bajo:
            self.canvas.context.move_to(self.lapiz.x, self.lapiz.y)
            self.canvas.context.set_line_width(3)
            self.lapiz.x = x
            self.lapiz.y = y
            self.canvas.context.line_to(self.lapiz.x, self.lapiz.y)
            self.canvas.context.stroke()

            # Actualiza la imagen si ha dibujado.
            if self.actualiza_automaticamente:
                self.actualizar_imagen()

        self.lapiz.x, self.lapiz.y = x, y
        
    def definir_color(self, color):
        r, g, b = color.obtener_componentes()
        self.canvas.context.set_source_rgb(r/255.0, g/255.0, b/255.0)

    def pintar_imagen(self, imagen, x=0, y=0):
        """Dibuja una imagen sobre la pizarra pero usando coordenadas de pantalla.

        Las coordenadas de pantalla tienen su origen en la esquina
        superior izquierda, no en el centro de la ventana.
        """
        imagen = pilas.motor.generar_imagen_cairo(imagen)
        w = imagen.get_width()
        h = imagen.get_height()
        self._pintar_parte_de_imagen(imagen, 0, 0, w, h, x, y)

    def pintar_grilla(self, grilla, x=0, y=0):
        imagen = pilas.motor.generar_imagen_cairo(grilla.image)
        w = grilla.cuadro_ancho
        h = grilla.cuadro_alto
        dx = grilla.cuadro * grilla.cuadro_ancho
        self._pintar_parte_de_imagen(imagen, dx, 0, w, h, x, y)

    def _pintar_parte_de_imagen(self, imagen_cairo, origen_x, origen_y, ancho, alto, x, y):
        """Dibuja una porcion de imagen sobre la pizarra pero usando coordenadas de pantalla.

        Los argumentos ``origen_x`` y ``origen_y`` indican la parte
        izquierda de la imagen que se descartará, ``ancho`` y ``alto``
        el tamaño de rectángulo que se desea leer y por último ``x`` e ``y``
        son las coordenadas destino de impresión.

        Ten en cuenta que las coordenadas de pantalla tienen su origen en la esquina
        superior izquierda, no en el centro de la ventana.
        """

        self.canvas.context.set_source_surface(imagen_cairo, x - origen_x, y - origen_y)
        self.canvas.context.rectangle(x, y, ancho, alto)
        self.canvas.context.fill()
        if self.actualiza_automaticamente:
            self.actualizar_imagen()

    def pintar(self, color=None):
        w = self.canvas.surface.get_width()
        h = self.canvas.surface.get_height()

        if color:
            self.definir_color(color)

        self.canvas.context.rectangle(0, 0, w, h)
        self.canvas.context.fill()
        if self.actualiza_automaticamente:
            self.actualizar_imagen()

    def escribir(self, texto, x=0, y=0, tamano=32, fuente="Arial"):
        "Pinta una cadena de texto con el color actual del trazo."
        self.canvas.context.move_to(x, y)
        self.canvas.context.set_font_size(tamano)
        self.canvas.context.select_font_face(fuente)
        self.canvas.context.text_path(texto)
        self.canvas.context.fill()
        if self.actualiza_automaticamente:
            self.actualizar_imagen()

    def dibujar_rectangulo(self, x, y, ancho, alto, pintar=True):
        self.canvas.context.rectangle(x, y, ancho, alto)

        if pintar:
            self.canvas.context.fill()

        self.canvas.context.stroke()
        if self.actualiza_automaticamente:
            self.actualizar_imagen()

    def dibujar_poligono(self, puntos):
        (x, y) = puntos.pop(0)
        (x, y) = pilas.utils.hacer_coordenada_mundo(x, y)
        self.canvas.context.move_to(x, y)

        for (x, y) in puntos:
            (x, y) = pilas.utils.hacer_coordenada_mundo(x, y)
            self.canvas.context.line_to(x, y)
        
        self.canvas.context.close_path()
        self.canvas.context.stroke()

        self.actualizar_imagen()





# -*- encoding: utf-8 -*-
# Pilas engine - A video game framework.
#
# Copyright 2010 - Hugo Ruscitti
# License: LGPLv3 (see http://www.gnu.org/licenses/lgpl.html)
#
# Website - http://www.pilas-engine.com.ar

from pilas.actores import Actor
import pilas


class Estrella(Actor):
    "Representa una estrella de color amarillo."

    def __init__(self, x=0, y=0):
        imagen = pilas.imagenes.cargar('estrella.png')
        Actor.__init__(self, imagen, x=x, y=y)
        self.rotacion = 0



# -*- encoding: utf-8 -*-
# Pilas engine - A video game framework.
#
# Copyright 2010 - Hugo Ruscitti
# License: LGPLv3 (see http://www.gnu.org/licenses/lgpl.html)
#
# Website - http://www.pilas-engine.com.ar


import math

import pilas
import utils

todos = []

Actor = pilas.motor.Actor

from mono import *
from ejes import *
from animado import Animado
from animacion import Animacion

from explosion import Explosion
from bomba import Bomba
from pingu import Pingu
from banana import Banana
from texto import Texto
from moneda import Moneda
from pizarra import Pizarra
from pelota import Pelota
from estrella import Estrella
from caja import Caja
from nave import Nave
from disparo import Disparo
from cursordisparo import CursorDisparo
from piedra import Piedra
from menu import Menu
from opcion import Opcion
from tortuga import Tortuga
from mapa import Mapa
from martian import Martian
from boton import Boton



import pilas
import sys

pilas.iniciar()

# paso 1: crear la pizarra
pizarra = pilas.actores.Pizarra()

# paso 2: comienza el dibujo personalizado con cairo:
import cairo

x1, y1 = (100, 100)
x2, y2 = (600, 300)
x3, y3 = (100, 400)

pizarra.canvas.context.curve_to(x1, y1, x2, y2, x3, y3)
pizarra.canvas.context.set_line_width(50)
pizarra.canvas.context.set_dash([10])
pizarra.canvas.context.stroke()

# paso 3: Decirle a la pizarra que se actualice.
pizarra.actualizar_imagen()

pilas.ejecutar()



"""
This file is part of the 'Elements' Project
Elements is a 2D Physics API for Python (supporting Box2D2)

Copyright (C) 2008, The Elements Team, <elements@linuxuser.at>

Home:  http://elements.linuxuser.at
IRC:   #elements on irc.freenode.org

Code:  http://www.assembla.com/wiki/show/elements
       svn co http://svn2.assembla.com/svn/elements                     

License:  GPLv3 | See LICENSE for the full text
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.              
"""
import pygame
from pygame.locals import *

import tools

COLOR_HEX_BLUE1 = "6491a4"
COLOR_HEX_BLUE2 = "9ec9ff"

class MenuItem:   
    # padding [px]: left, top, right, bottom
    padding = (5, 2, 5, 2)
 
    def empty(self, *args):
        pass
        
    def __init__(self, title, pos, userData, parent=None, callback=None):
        self.title = title
        self.userData = userData
        self.parent = parent
        self.childs = []
        
        if self.parent:
            self.visible = False
        else:
            self.visible = True
        
        if callback:
            self.callback = callback
        else:
            self.callback = self.empty

        # Create Surface and Stuff :)
        self.font = pygame.font.Font(None, 32)
        text = self.font.render(title, 1, (255,255,255))

        rx, ry, rw, rh = rect = text.get_rect()
        pl, pt, pr, pb = self.padding
        
        s1 = pygame.Surface((rw+pl+pr, rh+pt+pb))
        s1.fill(tools.hex2rgb(COLOR_HEX_BLUE1))
        s1.blit(text, (pl, pt))

        s2 = pygame.Surface((rw+pl+pr, rh+pt+pb))
        s2.fill(tools.hex2rgb(COLOR_HEX_BLUE2))
        s2.blit(text, (pl, pt))
        
        self.rect = s1.get_rect().move(pos)

        self.surface_inactive = s1
        self.surface_active = s2
        
    def pos_inside(self, pos):
        if not self.visible:
            return False
            
        x,y,w,h = self.rect
        px, py = pos
        
        if px > x and px < x+w and py > y and py < y+h:
            return True
        else:
            return False
                
class MenuClass:
    """ Important: Never delete an Item, just overwrite it if deleting,
        else the menuitem id's get messed up
    """
    # current active menu point it
    focus = False
    
    # each item is stored as MenuItem
    items = []    

    # where to start drawing
    start_at = (0, 0)
   
    # menubar properties
    height = 0        # px
    width = 0         # px (set in set_width)
    setWidth = False  # if width was set by hand (if not, increase width by adding stuff)
    
    def __init__(self):
        self.draw_at = self.start_at
        
    def set_width(self, width):
        self.setWidth = True
        self.width = width
        
    def addItem(self, title, callback=None, userData='', parent=None):
        # Get position for the Item
        if parent: draw_at = (0,0)
        else: draw_at = self.draw_at
        
        # Create Items
        M = MenuItem(title=title, pos=draw_at, userData=userData, parent=parent, callback=callback)
        self.items.append(M)
                
        # Set a new position
        x,y,w,h = M.rect
        x, y = self.draw_at
        
        if parent:
            # Set the info that the item has a child to the parent item
            self.items[parent-1].childs.append(len(self.items)-1) 

        else:
            # New next drawing position
            self.draw_at = (x+w, y)

            # Adjust the width of the menu bar
            if not self.setWidth:
                self.width = x+w     

        # Adjust the height of the menu bar
        if h > self.height: self.height = h + 2

        # Return array id of this item
        return len(self.items)
        
    def click(self, pos):
        """ Checks a click for menuitems and starts the callback if found
            
            Return: True if a menu item was found or hit the MenuBar, and False if not
        """
        focus_in = self.focus
        
        found = False
        for i in xrange(len(self.items)):
            item = self.items[i]
            if item.pos_inside(pos):
                found = True
                item.callback(item.title, item.userData)
                
                # Expand the menu if necessary
                if len(item.childs) > 0:
                    self.focus = i+1
                    
        # Close any opened menu windows if clicked somewhere else 
        if self.focus == focus_in:
            self.focus = False
            self.subwin_rect = (0,0,0,0)
            for item in self.items:
                if item.parent:
                    item.visible = False
                 
        # Check if click is inside menubar
        x,y = pos
        mx, my = self.start_at
        
        if found:
            return True
        else: 
            return False
        
    def draw(self, surface):
        """ Draw the menu with pygame on a given surface
        """
        s = pygame.Surface((self.width, self.height))
        s.fill(tools.hex2rgb(COLOR_HEX_BLUE1))

        surface.blit(s, (0,0))
        
        for i in xrange(len(self.items)):
            item = self.items[i]
            if not item.parent: 
                x,y,w,h = item.rect
                if self.focus == i+1:
                    surface.blit(item.surface_active, (x,y))
                else:
                    surface.blit(item.surface_inactive, (x,y))

        # If a menu item is open, draw that
        if self.focus:
            width = 0
            height = 0

            i = []
            for j in self.items:
                if j.parent == self.focus:
                    i.append(j)
                    x, y, w, h = j.rect
                    if w > width: width = w
                    height += h

            if len(i) > 0:
                s = pygame.Surface((width, height))
                s.fill(tools.hex2rgb(COLOR_HEX_BLUE1))

                # Parent Coordinates    
                px, py, pw, ph = self.items[self.focus-1].rect
                
                # y Counter
                y = 0
                
                for item in i:
                    item.visible = True
                    s.blit(item.surface_inactive, (0, y))

                    ix, iy, iw, ih = item.rect                        
                    if (ix, iy) == (0, 0):
                        item.rect = item.rect.move((px, y+ph))
                        ix, iy, iw, ih = item.rect                        

                    if iw < width:
                        item.rect = (ix,iy,width,ih)

                    y += ih
                
                surface.blit(s, (px,py+ph))
                self.subwin_rect = s.get_rect().move(px, py+ph)




# -*- encoding: utf-8 -*-
# pilas engine - a video game framework.
#
# copyright 2010 - hugo ruscitti
# license: lgplv3 (see http://www.gnu.org/licenses/lgpl.html)
#
# website - http://www.pilas-engine.com.ar
import random

class Grupo(list):
    """Un grupo es un contenedor que funciona como una lista normal, pero mejorada.

    Los grupos pueden contener actores, y permite que a todos los actores
    se los pueda tratar como uno.

    Por ejemplo si tienes un contenedor con 20 actores, podrías ampliar
    el tamaño de todos ellos juntos usando la sentencia::

            grupo = pilas.atajos.fabricar(pilas.actores.Mono, 20)
            grupo.escala = 2

    """

    def __getattr__(self, attr):
        """Esta funcion se asegura de que cada vez que se invoque a un metodo
        del grupo, en realidad, el grupo va a invocar a ese metodo pero
        en todos sus elementos. Algo asi como un map."""

        def map_a_todos(*k, **kw):
            for a in self:
                funcion = getattr(a, attr)
                funcion(*k, **kw)

        return map_a_todos

    def __setattr__(self, atributo, valor):
        for a in self:
            setattr(a, atributo, valor)

    def desordenar(self):
        for a in self:
            a.x = random.randint(-300, 300)
            a.y = random.randint(-200, 200)

    def limpiar(self):
        eliminar = list(self)
        for e in eliminar:
            e.eliminar()



import pilas

pilas.iniciar()

pizarra = pilas.actores.Pizarra()

def cuando_pulsa_el_boton(evento):
    pizarra.bajar_lapiz()

def cuando_deja_de_pulsar_el_boton(evento):
    pizarra.levantar_lapiz()

def cuando_mueve_el_mouse(evento):
    pizarra.mover_lapiz(evento.x, evento.y)
    
pizarra.levantar_lapiz()

pilas.eventos.click_de_mouse.conectar(cuando_pulsa_el_boton)
pilas.eventos.mueve_mouse.conectar(cuando_mueve_el_mouse)
pilas.eventos.termina_click.conectar(cuando_deja_de_pulsar_el_boton)

pilas.avisar("Use el mouse para dibujar.")
pilas.ejecutar()



import pilas
import sys

pilas.iniciar()
mono1 = pilas.actores.Mono()
mono2 = pilas.actores.Mono()

mono2.escala = 2
mono2.z = 100


mono1.aprender(pilas.habilidades.Arrastrable)

pilas.avisar("Hay dos actores, uno se puede arrastrar con el mouse.")
pilas.ejecutar()



import pilas

pilas.iniciar()
b = pilas.actores.Boton()


def cuando_pulsan_el_boton():
    pilas.avisar("Han pulsado el boton")

def cuando_pasa_sobre_el_boton():
    pilas.avisar("Pasa el mouse sobre el boton")


def cuando_deja_de_pulsar():
    pilas.avisar("")


b.conectar_presionado(cuando_pulsan_el_boton)
b.conectar_sobre(cuando_pasa_sobre_el_boton)
b.conectar_normal(cuando_deja_de_pulsar)

pilas.ejecutar()



import pilas
from PySFML import sf
from weakref import ref


pilas.iniciar()
pilas.avisar("Usa el mouse para mover al mono y ayudarlo a comer.")

# Creamos al mono que se puede arrastrar con el mouse
mono = pilas.actores.Mono()
mono.aprender(pilas.habilidades.Arrastrable)
mono.z = 1


# Creamos las bananas y las colocamos en una lista.
b1 = pilas.actores.Banana()
b1.x = 200

b2 = pilas.actores.Banana()
b2.y = 200

b3 = pilas.actores.Banana()
b3.y = -200

bananas = [b1, b2, b3]

# Creamos una bomba y la colocamos en una lista.
bomba = pilas.actores.Bomba()
bomba.x = -200

bombas = [bomba]


def comer_banana(mono, banana):
    banana.eliminar()
    mono.sonreir()

def hacer_explotar_una_bomba(mono, bomba):
    bomba.explotar()
    mono.gritar()

# Le indicamos a pilas que funcion tiene que ejecutar cuando
# se produzca una colision.
pilas.colisiones.agregar(mono, bananas, comer_banana)
pilas.colisiones.agregar(mono, bombas, hacer_explotar_una_bomba)

pilas.ejecutar()



import pygame

class FPS:

    def __init__(self, fps):
        self.antes = self.ahora = pygame.time.get_ticks()
        self.frecuencia = 1000.0 / fps
        self.t_fps = self.ahora
        self.rendimiento = 0
        self.cuadros_por_segundo = "??"

    def actualizar(self):
        retorno = 0
        self.ahora = pygame.time.get_ticks()

        dt = self.ahora - self.antes 

        while dt >= self.frecuencia:
            self.antes += self.frecuencia
            dt = self.ahora - self.antes 
            retorno += 1

            if self.ahora - self.t_fps > 1000.0:
                #print self.cuadros_por_segundo
                self.cuadros_por_segundo = str(self.rendimiento)
                self.t_fps += 1000.0
                self.rendimiento = 0
        else:
            pygame.time.delay(int(self.frecuencia - dt))
            returno = 1


        self.rendimiento += 1
        return retorno



import pilas
import sys

pilas.iniciar()
pizarra = pilas.actores.Pizarra()

pizarra.bajar_lapiz()

pizarra.mover_lapiz(100, 0)
pizarra.mover_lapiz(100, 100)
pizarra.mover_lapiz(0, 0)


#pilas.avisar("Una pizarra con algunas imagenes.")
pilas.ejecutar()



import pilas

# Colores principales.
negro = pilas.motor.Color(0, 0, 0)
blanco = pilas.motor.Color(255, 255, 255)
rojo = pilas.motor.Color(255, 0, 0)
verde = pilas.motor.Color(0, 255, 0)
azul = pilas.motor.Color(0, 0, 255)
gris = pilas.motor.Color(128, 128, 128)

# Colores secundarios
amarillo = pilas.motor.Color(255, 255, 0)
magenta = pilas.motor.Color(255, 0, 255)
cyan = pilas.motor.Color(0, 255, 255)
grisclaro = pilas.motor.Color(192, 192, 192)
grisoscuro = pilas.motor.Color(100, 100, 100)
verdeoscuro = pilas.motor.Color(0, 128, 0)
azuloscuro = pilas.motor.Color(0, 0, 128)
naranja = pilas.motor.Color(255, 200, 0)
rosa = pilas.motor.Color(255, 175, 175)
violeta = pilas.motor.Color(128, 0, 255)
marron = pilas.motor.Color(153, 102, 0)



# Colores transparentes
negro = pilas.motor.Color(0, 0, 0, 160)
blanco = pilas.motor.Color(255, 255, 255, 160)
rojo_transparente = pilas.motor.Color(255, 0, 0, 160)
verde_transparente = pilas.motor.Color(0, 255, 0, 160)
azul_transparente = pilas.motor.Color(0, 0, 255, 160)
gris_transparente = pilas.motor.Color(128, 128, 128, 160)



# -*- encoding: utf-8 -*-
# pilas engine - a video game framework.
#
# copyright 2010 - hugo ruscitti
# license: lgplv3 (see http://www.gnu.org/licenses/lgpl.html)
#
# website - http://www.pilas-engine.com.ar

import pilas

fabricar = pilas.actores.utils.fabricar

def crear_grupo(*k):
    return pilas.grupos.Grupo(k)

def definir_gravedad(x=0, y=-900):
    pilas.fisica.fisica.definir_gravedad(x, y)



import pilas
from random import random

class Juego(pilas.escenas.Escena):
    "Escena que Controla el juego"

    def __init__(self):        
        pilas.escenas.Escena.__init__(self) 
        pilas.actores.utils.eliminar_a_todos()
        pilas.fondos.Fondo('data/fondo.png')

        pilas.eventos.pulsa_tecla_escape.conectar(self.cuando_se_presione_escape)
        
        #creamos tablero
        self.crear_tablero()
        
        self.crear_pizarra() #servira para trazar lineas ganadoras
        
        #creamos fichas
        self.crear_fichas_1()
        self.crear_fichas_2()
        
        # turno = 0 para jugador 1
        # turno = 1 para jugador 2
        
        self.turno = int(random() * 2)
        
        # estado = -1 cuando estamos jugando
        # estado = 0 gana jugador 1
        # estado = 1 gano jugador 2
        # estado = 2 nadie gano
        
        self.estado = -1
        
        
        self.turno_o = pilas.imagenes.cargar('data/ficha1.png')
        self.turno_x = pilas.imagenes.cargar('data/ficha2.png')
        if self.turno == 0:
            self.turno_actual = pilas.actores.Actor(self.turno_o, -163, 80)
            self.turno_actual.escala = 0.5
        else:
            self.turno_actual = pilas.actores.Actor(self.turno_x, -163, 80)
            self.turno_actual.escala = 0.5
        
        self.conectar_fichas1_con_eventos()
        self.conectar_fichas2_con_eventos()
        
    def cuando_se_presione_escape(self, *k, **kv):
        "Regresa al menu principal"
        import escena_menu
        escena_menu.EscenaMenu()
    

    def crear_pizarra(self):
        self.pizarra = pilas.actores.Pizarra()
    

    def pintar_linea_horizontal(self, h):
        self.pizarra.levantar_lapiz()
        
        self.pizarra.mover_lapiz(0,h)
        
        self.pizarra.bajar_lapiz()
        
        self.pizarra.definir_color(pilas.colores.cyan)
        self.pizarra.mover_lapiz(-100,h)
        self.pizarra.mover_lapiz(100,h)

    
    
    
    def pintar_linea_diagonal_1(self):
        self.pizarra.levantar_lapiz()
        
        self.pizarra.mover_lapiz(-84,84)
        
        self.pizarra.bajar_lapiz()
        
        self.pizarra.definir_color(pilas.colores.cyan)
        self.pizarra.mover_lapiz(84,-84) 
        
    def pintar_linea_diagonal_2(self):
        self.pizarra.levantar_lapiz()
        
        self.pizarra.mover_lapiz(84,84)
        
        self.pizarra.bajar_lapiz()
        
        self.pizarra.definir_color(pilas.colores.cyan)
        self.pizarra.mover_lapiz(-84,-84) 
        
    def pintar_linea_vertical(self, v):
        self.pizarra.levantar_lapiz()
        
        self.pizarra.mover_lapiz(v,0)
        
        self.pizarra.bajar_lapiz()
        
        self.pizarra.definir_color(pilas.colores.cyan)
        self.pizarra.mover_lapiz(v,-100)
        self.pizarra.mover_lapiz(v,100)
    
    def crear_tablero(self):
        self.tablero_matriz = [[-1, -1, -1],[-1, -1, -1],[-1, -1, -1]]
        
        # esta matriz se utiliza para saber que ficha se encuentra
        # en cada lugar o si es una casilla vacia.
        # 0 = ficha 1 
        # 1 = ficha 2
        # -1 = no hay ficha
        
    def crear_fichas_1(self):
        
        self.ruta_ficha_vacia = 'data/ficha_vacia.png'
        self.ruta_ficha_1 = 'data/ficha1.png'
        
        self.ficha1_1 = pilas.actores.Boton(-84, 84,  self.ruta_ficha_vacia,  self.ruta_ficha_1, self.ruta_ficha_1)
        self.ficha1_2 = pilas.actores.Boton(0, 84,  self.ruta_ficha_vacia,  self.ruta_ficha_1, self.ruta_ficha_1)
        self.ficha1_3 = pilas.actores.Boton(84, 84,  self.ruta_ficha_vacia,  self.ruta_ficha_1, self.ruta_ficha_1)
        self.ficha1_4 = pilas.actores.Boton(-84, 0,  self.ruta_ficha_vacia,  self.ruta_ficha_1, self.ruta_ficha_1)
        self.ficha1_5 = pilas.actores.Boton(0, 0,  self.ruta_ficha_vacia,  self.ruta_ficha_1, self.ruta_ficha_1)
        self.ficha1_6 = pilas.actores.Boton(84, 0,  self.ruta_ficha_vacia,  self.ruta_ficha_1, self.ruta_ficha_1)
        self.ficha1_7 = pilas.actores.Boton(-84, -84,  self.ruta_ficha_vacia,  self.ruta_ficha_1, self.ruta_ficha_1)
        self.ficha1_8 = pilas.actores.Boton(0, -84,  self.ruta_ficha_vacia,  self.ruta_ficha_1, self.ruta_ficha_1)
        self.ficha1_9 = pilas.actores.Boton(84, -84,  self.ruta_ficha_vacia,  self.ruta_ficha_1, self.ruta_ficha_1)
        
        
    def crear_fichas_2(self):
        
        self.ruta_ficha_vacia = 'data/ficha_vacia.png'
        self.ruta_ficha_2 = 'data/ficha2.png'
        
        self.ficha2_1 = pilas.actores.Boton(-84, 84,  self.ruta_ficha_vacia, self.ruta_ficha_2, self.ruta_ficha_2)
        self.ficha2_2 = pilas.actores.Boton(0, 84,  self.ruta_ficha_vacia, self.ruta_ficha_2, self.ruta_ficha_2)
        self.ficha2_3 = pilas.actores.Boton(84, 84,  self.ruta_ficha_vacia, self.ruta_ficha_2, self.ruta_ficha_2)
        self.ficha2_4 = pilas.actores.Boton(-84, 0,  self.ruta_ficha_vacia, self.ruta_ficha_2, self.ruta_ficha_2)
        self.ficha2_5 = pilas.actores.Boton(0, 0,  self.ruta_ficha_vacia, self.ruta_ficha_2, self.ruta_ficha_2)
        self.ficha2_6 = pilas.actores.Boton(84, 0,  self.ruta_ficha_vacia, self.ruta_ficha_2, self.ruta_ficha_2)
        self.ficha2_7 = pilas.actores.Boton(-84, -84,  self.ruta_ficha_vacia, self.ruta_ficha_2, self.ruta_ficha_2)
        self.ficha2_8 = pilas.actores.Boton(0, -84,  self.ruta_ficha_vacia, self.ruta_ficha_2, self.ruta_ficha_2)
        self.ficha2_9 = pilas.actores.Boton(84, -84,  self.ruta_ficha_vacia, self.ruta_ficha_2, self.ruta_ficha_2)
        
    
    
    
    
    
    
    def cambiar_visor_turno_o(self):
        self.turno_actual.definir_imagen(self.turno_o)
        
    def cambiar_visor_turno_x(self):
        self.turno_actual.definir_imagen(self.turno_x)
        
    def conectar_fichas1_con_eventos(self):
        self.ficha1_1.conectar_presionado(self.press_ficha1_1)
        self.ficha1_1.conectar_presionado(self.analizar_ganador)
        
        self.ficha1_2.conectar_presionado(self.press_ficha1_2)
        self.ficha1_2.conectar_presionado(self.analizar_ganador)
        
        self.ficha1_3.conectar_presionado(self.press_ficha1_3)
        self.ficha1_3.conectar_presionado(self.analizar_ganador)
        
        self.ficha1_4.conectar_presionado(self.press_ficha1_4)
        self.ficha1_4.conectar_presionado(self.analizar_ganador)
        
        self.ficha1_5.conectar_presionado(self.press_ficha1_5)
        self.ficha1_5.conectar_presionado(self.analizar_ganador)
        
        self.ficha1_6.conectar_presionado(self.press_ficha1_6)
        self.ficha1_6.conectar_presionado(self.analizar_ganador)
        
        self.ficha1_7.conectar_presionado(self.press_ficha1_7)
        self.ficha1_7.conectar_presionado(self.analizar_ganador)
        
        self.ficha1_8.conectar_presionado(self.press_ficha1_8)
        self.ficha1_8.conectar_presionado(self.analizar_ganador)
        
        self.ficha1_9.conectar_presionado(self.press_ficha1_9)
        self.ficha1_9.conectar_presionado(self.analizar_ganador)
        
    
    def conectar_fichas2_con_eventos(self):
        self.ficha2_1.conectar_presionado(self.press_ficha2_1)
        self.ficha2_1.conectar_presionado(self.analizar_ganador)
        
        self.ficha2_2.conectar_presionado(self.press_ficha2_2)
        self.ficha2_2.conectar_presionado(self.analizar_ganador)
        
        self.ficha2_3.conectar_presionado(self.press_ficha2_3)
        self.ficha2_3.conectar_presionado(self.analizar_ganador)
        
        self.ficha2_4.conectar_presionado(self.press_ficha2_4)
        self.ficha2_4.conectar_presionado(self.analizar_ganador)
        
        self.ficha2_5.conectar_presionado(self.press_ficha2_5)
        self.ficha2_5.conectar_presionado(self.analizar_ganador)
        
        self.ficha2_6.conectar_presionado(self.press_ficha2_6)
        self.ficha2_6.conectar_presionado(self.analizar_ganador)
        
        self.ficha2_7.conectar_presionado(self.press_ficha2_7)
        self.ficha2_7.conectar_presionado(self.analizar_ganador)
        
        self.ficha2_8.conectar_presionado(self.press_ficha2_8)
        self.ficha2_8.conectar_presionado(self.analizar_ganador)
        
        self.ficha2_9.conectar_presionado(self.press_ficha2_9)
        self.ficha2_9.conectar_presionado(self.analizar_ganador)
    
    
    # metodos para cada ficha 1
    
    # controlan si hay una casilla vacia, para poner una ficha
    # controla el visor de turnos y los turnos
    
    def press_ficha1_1(self):
        if self.estado == -1:
            if self.turno == 0:
                if self.tablero_matriz[0][0] == -1:
                    self.tablero_matriz[0][0] = 0 
                    self.ficha1_1.pintar_presionado()
                    self.ficha1_1.escala = 0.8
                    self.ficha1_1.escala = pilas.interpolar([1], duracion=0.1)
                    self.turno = 1
                    self.cambiar_visor_turno_x()
                    
    def press_ficha1_2(self):
        if self.estado == -1:
            if self.turno == 0:
                if self.tablero_matriz[0][1] == -1:
                    self.tablero_matriz[0][1] = 0 
                    self.ficha1_2.pintar_presionado()
                    self.ficha1_2.escala = 0.8
                    self.ficha1_2.escala = pilas.interpolar([1], duracion=0.1)
                    self.turno = 1
                    self.cambiar_visor_turno_x()
    
    def press_ficha1_3(self):
        if self.estado == -1:
            if self.turno == 0:
                if self.tablero_matriz[0][2] == -1:
                    self.tablero_matriz[0][2] = 0 
                    self.ficha1_3.pintar_presionado()
                    self.ficha1_3.escala = 0.8
                    self.ficha1_3.escala = pilas.interpolar([1], duracion=0.1)
                    self.turno = 1
                    self.cambiar_visor_turno_x()
    
    def press_ficha1_4(self):
        if self.estado == -1:
            if self.turno == 0:
                if self.tablero_matriz[1][0] == -1:
                    self.tablero_matriz[1][0] = 0 
                    self.ficha1_4.pintar_presionado()
                    self.ficha1_4.escala = 0.8
                    self.ficha1_4.escala = pilas.interpolar([1], duracion=0.1)
                    self.turno = 1
                    self.cambiar_visor_turno_x()
    
    def press_ficha1_5(self):
        if self.estado == -1:
            if self.turno == 0:
                if self.tablero_matriz[1][1] == -1:
                    self.tablero_matriz[1][1] = 0 
                    self.ficha1_5.pintar_presionado()
                    self.ficha1_5.escala = 0.8
                    self.ficha1_5.escala = pilas.interpolar([1], duracion=0.1)
                    self.turno = 1
                    self.cambiar_visor_turno_x()
    
    def press_ficha1_6(self):
        if self.estado == -1:
            if self.turno == 0:
                if self.tablero_matriz[1][2] == -1:
                    self.tablero_matriz[1][2] = 0 
                    self.ficha1_6.pintar_presionado()
                    self.ficha1_6.escala = 0.8
                    self.ficha1_6.escala = pilas.interpolar([1], duracion=0.1)
                    self.turno = 1
                    self.cambiar_visor_turno_x()
    
    def press_ficha1_7(self):
        if self.estado == -1:
            if self.turno == 0:
                if self.tablero_matriz[2][0] == -1:
                    self.tablero_matriz[2][0] = 0 
                    self.ficha1_7.pintar_presionado()
                    self.ficha1_7.escala = 0.8
                    self.ficha1_7.escala = pilas.interpolar([1], duracion=0.1)
                    self.turno = 1
                    self.cambiar_visor_turno_x()
    
    def press_ficha1_8(self):
        if self.estado == -1:
            if self.turno == 0:
                if self.tablero_matriz[2][1] == -1:
                    self.tablero_matriz[2][1] = 0 
                    self.ficha1_8.pintar_presionado()
                    self.ficha1_8.escala = 0.8
                    self.ficha1_8.escala = pilas.interpolar([1], duracion=0.1)
                    self.turno = 1
                    self.cambiar_visor_turno_x()
    
    def press_ficha1_9(self):
        if self.estado == -1:
            if self.turno == 0:
                if self.tablero_matriz[2][2] == -1:
                    self.tablero_matriz[2][2] = 0 
                    self.ficha1_9.pintar_presionado()
                    self.ficha1_9.escala = 0.8
                    self.ficha1_9.escala = pilas.interpolar([1], duracion=0.1)
                    self.turno = 1
                    self.cambiar_visor_turno_x()
    
    
    
    
    
    
    
    
    
    #metodos para cada ficha 2
    
    def press_ficha2_1(self):
        if self.estado == -1:
            if self.turno == 1:
                if self.tablero_matriz[0][0] == -1:
                    self.tablero_matriz[0][0] = 1 
                    self.ficha2_1.pintar_presionado()
                    self.ficha2_1.escala = 0.8
                    self.ficha2_1.escala = pilas.interpolar([1], duracion=0.1)
                    self.turno = 0
                    self.cambiar_visor_turno_o()

    def press_ficha2_2(self):
        if self.estado == -1:
            if self.turno == 1:
                if self.tablero_matriz[0][1] == -1:
                    self.tablero_matriz[0][1] = 1 
                    self.ficha2_2.pintar_presionado()
                    self.ficha2_2.escala = 0.8
                    self.ficha2_2.escala = pilas.interpolar([1], duracion=0.1)
                    self.turno = 0
                    self.cambiar_visor_turno_o()

    def press_ficha2_3(self):
        if self.estado == -1:
            if self.turno == 1:
                if self.tablero_matriz[0][2] == -1:
                    self.tablero_matriz[0][2] = 1 
                    self.ficha2_3.pintar_presionado()
                    self.ficha2_3.escala = 0.8
                    self.ficha2_3.escala = pilas.interpolar([1], duracion=0.1)
                    self.turno = 0
                    self.cambiar_visor_turno_o()
    
    def press_ficha2_4(self):
        if self.estado == -1:
            if self.turno == 1:
                if self.tablero_matriz[1][0] == -1:
                    self.tablero_matriz[1][0] = 1 
                    self.ficha2_4.pintar_presionado()
                    self.ficha2_4.escala = 0.8
                    self.ficha2_4.escala = pilas.interpolar([1], duracion=0.1)
                    self.turno = 0
                    self.cambiar_visor_turno_o()
    
    def press_ficha2_5(self):
        if self.estado == -1:
            if self.turno == 1:
                if self.tablero_matriz[1][1] == -1:
                    self.tablero_matriz[1][1] = 1
                    self.ficha2_5.pintar_presionado()
                    self.ficha2_5.escala = 0.8
                    self.ficha2_5.escala = pilas.interpolar([1], duracion=0.1)
                    self.turno = 0
                    self.cambiar_visor_turno_o()
    
    def press_ficha2_6(self):
        if self.estado == -1:
            if self.turno == 1:
                if self.tablero_matriz[1][2] == -1:
                    self.tablero_matriz[1][2] = 1 
                    self.ficha2_6.pintar_presionado()
                    self.ficha2_6.escala = 0.8
                    self.ficha2_6.escala = pilas.interpolar([1], duracion=0.1)
                    self.turno = 0
                    self.cambiar_visor_turno_o()
    
    def press_ficha2_7(self):
        if self.estado == -1:
            if self.turno == 1:
                if self.tablero_matriz[2][0] == -1:
                    self.tablero_matriz[2][0] = 1 
                    self.ficha2_7.pintar_presionado()
                    self.ficha2_7.escala = 0.8
                    self.ficha2_7.escala = pilas.interpolar([1], duracion=0.1)
                    self.turno = 0
                    self.cambiar_visor_turno_o()
    
    def press_ficha2_8(self):
        if self.estado == -1:
            if self.turno == 1:
                if self.tablero_matriz[2][1] == -1:
                    self.tablero_matriz[2][1] = 1 
                    self.ficha2_8.pintar_presionado()
                    self.ficha2_8.escala = 0.8
                    self.ficha2_8.escala = pilas.interpolar([1], duracion=0.1)
                    self.turno = 0
                    self.cambiar_visor_turno_o()
    
    def press_ficha2_9(self):
        if self.estado == -1:
            if self.turno == 1:
                if self.tablero_matriz[2][2] == -1:
                    self.tablero_matriz[2][2] = 1 
                    self.ficha2_9.pintar_presionado()
                    self.ficha2_9.escala = 0.8
                    self.ficha2_9.escala = pilas.interpolar([1], duracion=0.1)
                    self.turno = 0
                    self.cambiar_visor_turno_o()
    
    
    

    
    

    # comprueba si a ocurrido tres en raya o empate
    def nadie_gano(self):
        for i in range(3):
            for j in range(3):
                if self.tablero_matriz[i][j] == -1:
                    return False
        return True
        
    def analizar_ganador(self):
        
        self.posicion_texto = [0, -180]
        
        ## gano player 1
        
        if (self.tablero_matriz[0][0] == self.tablero_matriz[0][1] == self.tablero_matriz[0][2] == 0):
            if self.estado == -1:
                t = pilas.actores.Texto("Gana Jugador 1!", 
                                        self.posicion_texto[0], 
                                        self.posicion_texto[1])
                self.estado = 0
                self.pintar_linea_horizontal(84)
                t.color = (pilas.colores.blanco)
                
        elif (self.tablero_matriz[0][0] == self.tablero_matriz[1][0] == self.tablero_matriz[2][0] == 0):
            if self.estado == -1:
                t = pilas.actores.Texto("Gana Jugador 1!", 
                                        self.posicion_texto[0], 
                                        self.posicion_texto[1])
                self.estado = 0
                self.pintar_linea_vertical(-84)
                t.color = (pilas.colores.blanco)

        if (self.tablero_matriz[1][0] == self.tablero_matriz[1][1] == self.tablero_matriz[1][2] == 0):
            if self.estado == -1:
                t = pilas.actores.Texto("Gana Jugador 1!", 
                                        self.posicion_texto[0], 
                                        self.posicion_texto[1])
                self.estado = 0
                self.pintar_linea_horizontal(0)
                t.color = (pilas.colores.blanco)
        
        elif (self.tablero_matriz[0][1] == self.tablero_matriz[1][1] == self.tablero_matriz[2][1] == 0):
            if self.estado == -1:
                t = pilas.actores.Texto("Gana Jugador 1!", 
                                        self.posicion_texto[0], 
                                        self.posicion_texto[1])
                self.estado = 0
                self.pintar_linea_vertical(0)
                t.color = (pilas.colores.blanco)

        if (self.tablero_matriz[0][2] == self.tablero_matriz[1][2] == self.tablero_matriz[2][2] == 0):
            if self.estado == -1:
                t = pilas.actores.Texto("Gana Jugador 1!", 
                                        self.posicion_texto[0], 
                                        self.posicion_texto[1])
                self.estado = 0
                self.pintar_linea_vertical(84)
                t.color = (pilas.colores.blanco)

        elif (self.tablero_matriz[2][0] == self.tablero_matriz[2][1] == self.tablero_matriz[2][2] == 0):
            if self.estado == -1:
                t = pilas.actores.Texto("Gana Jugador 1!", 
                                        self.posicion_texto[0], 
                                        self.posicion_texto[1])
                self.estado = 0
                self.pintar_linea_horizontal(-84)
                t.color = (pilas.colores.blanco)

        if (self.tablero_matriz[0][0] == self.tablero_matriz[1][1] == self.tablero_matriz[2][2] == 0):
            if self.estado == -1:
                t = pilas.actores.Texto("Gana Jugador 1!", 
                                        self.posicion_texto[0], 
                                        self.posicion_texto[1])
                self.estado = 0
                self.pintar_linea_diagonal_1()
                t.color = (pilas.colores.blanco)
                
        if (self.tablero_matriz[0][2] == self.tablero_matriz[1][1] == self.tablero_matriz[2][0] == 0):
            if self.estado == -1:
                t = pilas.actores.Texto("Gana Jugador 1!", 
                                        self.posicion_texto[0], 
                                        self.posicion_texto[1])
                self.estado = 0
                self.pintar_linea_diagonal_2()
                t.color = (pilas.colores.blanco)
	
	
	    ## gano player 2
        
        if (self.tablero_matriz[0][0] == self.tablero_matriz[0][1] == self.tablero_matriz[0][2] == 1):
            if self.estado == -1:
                t = pilas.actores.Texto("Gana Jugador 2!", 
                                        self.posicion_texto[0], 
                                        self.posicion_texto[1])
                self.estado = 1
                self.pintar_linea_horizontal(84)
                t.color = (pilas.colores.blanco)
                
        elif (self.tablero_matriz[0][0] == self.tablero_matriz[1][0] == self.tablero_matriz[2][0] == 1):
            if self.estado == -1:
                t = pilas.actores.Texto("Gana Jugador 2!", 
                                        self.posicion_texto[0], 
                                        self.posicion_texto[1])
                self.estado = 1
                self.pintar_linea_vertical(-84)
                t.color = (pilas.colores.blanco)

        if (self.tablero_matriz[1][0] == self.tablero_matriz[1][1] == self.tablero_matriz[1][2] == 1):
            if self.estado == -1:
                t = pilas.actores.Texto("Gana Jugador 2!", 
                                        self.posicion_texto[0], 
                                        self.posicion_texto[1])
                self.estado = 1
                self.pintar_linea_horizontal(0)
        
        elif (self.tablero_matriz[0][1] == self.tablero_matriz[1][1] == self.tablero_matriz[2][1] == 1):
            if self.estado == -1:
                t = pilas.actores.Texto("Gana Jugador 2!", 
                                        self.posicion_texto[0], 
                                        self.posicion_texto[1])
                self.estado = 1
                self.pintar_linea_vertical(0)
                t.color = (pilas.colores.blanco)

        if (self.tablero_matriz[0][2] == self.tablero_matriz[1][2] == self.tablero_matriz[2][2] == 1):
            if self.estado == -1:
                t = pilas.actores.Texto("Gana Jugador 2!", 
                                        self.posicion_texto[0], 
                                        self.posicion_texto[1])
                self.estado = 1
                self.pintar_linea_vertical(84)
                t.color = (pilas.colores.blanco)

        elif (self.tablero_matriz[2][0] == self.tablero_matriz[2][1] == self.tablero_matriz[2][2] == 1):
            if self.estado == -1:
                t = pilas.actores.Texto("Gana Jugador 2!", 
                                        self.posicion_texto[0], 
                                        self.posicion_texto[1])
                self.estado = 1
                self.pintar_linea_horizontal(-84)
                t.color = (pilas.colores.blanco)

        if (self.tablero_matriz[0][0] == self.tablero_matriz[1][1] == self.tablero_matriz[2][2] == 1):
            if self.estado == -1:
                t = pilas.actores.Texto("Gana Jugador 2!", 
                                        self.posicion_texto[0], 
                                        self.posicion_texto[1])
                self.estado = 1
                self.pintar_linea_diagonal_1()
                t.color = (pilas.colores.blanco)
        if (self.tablero_matriz[0][2] == self.tablero_matriz[1][1] == self.tablero_matriz[2][0] == 1):
            if self.estado == -1:
                t = pilas.actores.Texto("Gana Jugador 2!", 
                                        self.posicion_texto[0], 
                                        self.posicion_texto[1])
                self.estado = 1
                self.pintar_linea_diagonal_2()
                t.color = (pilas.colores.blanco)
                
        temp_nadie_gana = self.nadie_gano()


        if temp_nadie_gana == True:
            if self.estado == -1:
                t = pilas.actores.Texto("Nadie ha ganado, vuelve a intentarlo!",
                                         self.posicion_texto[0], 
                                         self.posicion_texto[1])
                self.estado = 2
                t.color = (pilas.colores.blanco)





# -*- encoding: utf-8 -*-
# Pilas engine - A video game framework.
#
# Copyright 2010 - Hugo Ruscitti
# License: LGPLv3 (see http://www.gnu.org/licenses/lgpl.html)
#
# Website - http://www.pilas-engine.com.ar

import pilas

Texto = pilas.motor.Texto



# -*- encoding: utf-8 -*-
# Pilas engine - A video game framework.
#
# Copyright 2010 - Hugo Ruscitti
# License: LGPLv3 (see http://www.gnu.org/licenses/lgpl.html)
#
# Website - http://www.pilas-engine.com.ar

import pilas
from pilas.actores import Animacion

class Explosion(Animacion):
    "Representa una explosion para una bomba, dinamita etc..."

    def __init__(self, x=0, y=0):
        Animacion.__init__(self, pilas.imagenes.Grilla("explosion.png", 7), x=x, y=y)
        self.sonido_explosion = pilas.sonidos.cargar("explosion.wav")
        self.sonido_explosion.reproducir()



# -*- encoding: utf-8 -*-
# Pilas engine - A video game framework.
#
# Copyright 2010 - Hugo Ruscitti
# License: LGPLv3 (see http://www.gnu.org/licenses/lgpl.html)
#
# Website - http://www.pilas-engine.com.ar

from pilas.actores import Actor
from pilas.grupo import Grupo
import pilas

DEMORA = 14

class Menu(Actor):
    "Representa un bloque que tiene fisica como una caja."

    def __init__(self, opciones, x=0, y=0):
        self.opciones_como_actores = []
        Actor.__init__(self, "invisible.png", x=x, y=y)
        self._verificar_opciones(opciones)
        self.crear_texto_de_las_opciones(opciones)
        self.opciones = opciones
        self.seleccionar_primer_opcion()
        self.opcion_actual = 0
        # contador para evitar la repeticion de teclas
        self.demora_al_responder = 0

    def crear_texto_de_las_opciones(self, opciones):
        "Genera un actor por cada opcion del menu."

        for indice, (texto, funcion) in enumerate(opciones):
            y = self.y - indice * 50
            opciones = pilas.actores.Opcion(texto, y=y, funcion_a_invocar=funcion)

            self.opciones_como_actores.append(opciones)

    def seleccionar_primer_opcion(self):
        if self.opciones_como_actores:
            self.opciones_como_actores[0].resaltar()

    def _verificar_opciones(self, opciones):
        "Se asegura de que la lista este bien definida."

        for x in opciones:
            if not isinstance(x, tuple) or len(x) != 2:
                raise Exception("Opciones incorrectas, cada opcion tiene que ser una tupla.")

    def actualizar(self):
        "Se ejecuta de manera periodica."

        if self.demora_al_responder < 0:
            if pilas.mundo.control.boton:
                self.seleccionar_opcion_actual()
                self.demora_al_responder = DEMORA

            if pilas.mundo.control.abajo:
                self.mover_cursor(1)
                self.demora_al_responder = DEMORA
            elif pilas.mundo.control.arriba:
                self.mover_cursor(-1)
                self.demora_al_responder = DEMORA

        self.demora_al_responder -= 1

    def seleccionar_opcion_actual(self):
        opcion = self.opciones_como_actores[self.opcion_actual]
        opcion.seleccionar()

    def mover_cursor(self, delta):
        # Deja como no-seleccionada la opcion actual.
        self.opciones_como_actores[self.opcion_actual].resaltar(False)

        # Se asegura que las opciones esten entre 0 y 'cantidad de opciones'.
        self.opcion_actual += delta
        self.opcion_actual %= len(self.opciones_como_actores)

        # Selecciona la opcion nueva.
        self.opciones_como_actores[self.opcion_actual].resaltar()
    
    def __setattr__(self, atributo, valor):
        # Intenta propagar la accion a los actores del grupo.
        try:
            for x in self.opciones_como_actores:
                setattr(x, atributo, valor)
        except AttributeError:
            pass

        Actor.__setattr__(self, atributo, valor)



# -*- encoding: utf-8 -*-
# Pilas engine - A video game framework.
#
# Copyright 2010 - Hugo Ruscitti
# License: LGPLv3 (see http://www.gnu.org/licenses/lgpl.html)
#
# Website - http://www.pilas-engine.com.ar

import pilas
from pilas.simbolos import *

class Control:
    """Representa un control de teclado sencillo.

    Este objeto permite acceder al estado del teclado usando
    atributos.

    Por ejemplo, con este objeto, para saber si el usuario
    está pulsando el direccional hacia la izquierda de
    puede ejecutar::

        if pilas.control.izquierda:
            print 'Ha pulsado hacia la izquierda'
    """

    def __init__(self):
        self.izquierda = False
        self.derecha = False
        self.arriba = False
        self.abajo = False
        self.boton = False

    def actualizar(self):
        self.izquierda = pilas.motor.pulsa_tecla(IZQUIERDA)
        self.derecha = pilas.motor.pulsa_tecla(DERECHA)
        self.arriba = pilas.motor.pulsa_tecla(ARRIBA)
        self.abajo = pilas.motor.pulsa_tecla(ABAJO)
        self.boton = pilas.motor.pulsa_tecla(BOTON) or pilas.motor.pulsa_tecla(SELECCION)



# -*- encoding: utf-8 -*-
# Pilas engine - A video game framework.
#
# Copyright 2010 - Hugo Ruscitti
# License: LGPLv3 (see http://www.gnu.org/licenses/lgpl.html)
#
# Website - http://www.pilas-engine.com.ar

import pilas
from pilas.actores import Actor


class Tortuga(Actor):
    "Representa una tortuga que se mueve por la pantalla como la tortuga de Logo."

    def __init__(self, x=0, y=0, dibuja=True):
        self.pizarra = pilas.actores.Pizarra()

        imagen = pilas.imagenes.cargar('tortuga.png')
        Actor.__init__(self, imagen, x=x, y=y)

        if dibuja:
            self.bajalapiz()
        else:
            self.subelapiz()

        self.rotacion = 0
        self.velocidad = 6

    def avanzar(self, pasos):
        self.hacer_luego(pilas.comportamientos.Avanzar(pasos, self.velocidad))

    def giraderecha(self, delta):
        self.hacer_luego(pilas.comportamientos.Girar(abs(delta), self.velocidad))

    def giraizquierda(self, delta):
        self.hacer_luego(pilas.comportamientos.Girar(-abs(delta), self.velocidad))

    def actualizar(self):
        self.pizarra.mover_lapiz(self.x, self.y)

    def bajalapiz(self):
        self.hacer_luego(pilas.comportamientos.BajarLapiz())

    def subelapiz(self):
        self.hacer_luego(pilas.comportamientos.SubirLapiz())

    def pon_color(self, color):
        self.hacer_luego(pilas.comportamientos.CambiarColor(color))

    # Alias de metodos
    av = avanzar
    gd = giraderecha
    gi = giraizquierda
    bl = bajalapiz
    sl = subelapiz
    pc = pon_color


    def get_color(self):
        return self.pizarra.color

    def set_color(self, color):
        self.pizarra.definir_color(color)

    color = property(get_color, set_color)

    def limpiar(self):
        self.pizarra.limpiar()

    def pintar(self, color=None):
        self.pizarra.pintar(color)




# -*- encoding: utf-8 -*-
# Pilas engine - A video game framework.
#
# Copyright 2010 - Hugo Ruscitti
# License: LGPLv3 (see http://www.gnu.org/licenses/lgpl.html)
#
# Website - http://www.pilas-engine.com.ar

import random
import pilas

def ordenar_actores_por_valor_z():
    "Ordena todos los actores para que se impriman con 'z' como criterio de orden."
    pilas.actores.todos.sort()

def insertar_como_nuevo_actor(actor):
    "Coloca a un actor en la lista de actores a imprimir en pantalla."
    pilas.actores.todos.append(actor)
    
def eliminar_un_actor(actor):
    try:
        pilas.actores.todos.remove(actor)
    except ValueError:
        #TODO: quitar este silenciador de excepcion.
        pass

def eliminar_a_todos():
    a_eliminar = list(pilas.actores.todos)

    for x in a_eliminar:
        x.eliminar()

def obtener_actor_en(x, y):
    "Intenta obtener el actor mas cerca de la pantalla (z mas pequeño) en la posición (x, y)"

    # Busca el objeto que colisiones ordenando en sentido inverso.
    for sprite in pilas.actores.todos[::-1]:
        if sprite.colisiona_con_un_punto(x, y):
            return sprite

    return None


def fabricar(clase, cantidad=1, posiciones_al_azar=True, *k, **kv):
    "Genera muchas intancias de objetos asignando posiciones aleatorias."

    objetos_creados = []

    for x in range(cantidad):
        if posiciones_al_azar:
            x = random.randint(-300, 300)
            y = random.randint(-200, 200)
        else:
            x = 0
            y = 0

        nuevo = clase(*k,x=x, y=y,  **kv )
        objetos_creados.append(nuevo)

    return pilas.grupo.Grupo(objetos_creados)



# -*- encoding: utf-8 -*-
# Pilas engine - A video game framework.
#
# Copyright 2010 - Hugo Ruscitti
# License: LGPLv3 (see http://www.gnu.org/licenses/lgpl.html)
#
# Website - http://www.pilas-engine.com.ar

import pilas
from pilas.actores import Actor

class Banana(Actor):
    "Representa una explosion para una bomba, dinamita etc..."

    def __init__(self, x=0, y=0):
        self.animacion = pilas.imagenes.Grilla("banana.png", 2)
        Actor.__init__(self, x=x, y=y)
        self.definir_cuadro(0)

    def definir_cuadro(self, indice):
        self.animacion.definir_cuadro(indice)
        self.animacion.asignar(self)

    def abrir(self):
        self.definir_cuadro(1)

    def cerrar(self):
        self.definir_cuadro(0)



import pilas

pilas.iniciar()
mono = pilas.actores.Mono()

pasos = 100

mono.x = -50
mono.y = -50

mono.hacer(pilas.comportamientos.Avanzar(0, pasos), True)
mono.hacer(pilas.comportamientos.Avanzar(90, pasos), True)
mono.hacer(pilas.comportamientos.Avanzar(180, pasos), True)
mono.hacer(pilas.comportamientos.Girar(360, 10), True)
mono.hacer(pilas.comportamientos.Avanzar(270, pasos), True)

pilas.avisar("Movimiento mediante comportamientos.")
pilas.ejecutar()



import pilas
import sys

pilas.iniciar(usar_motor='pygame')
mono = pilas.actores.Mono()
mono.aprender(pilas.habilidades.Arrastrable)

pilas.avisar("Use el puntero del mouse para arrastrar al actor (pygame).")
pilas.ejecutar()



# -*- encoding: utf-8 -*-
# Pilas engine - A video game framework.
#
# Copyright 2010 - Hugo Ruscitti
# License: LGPLv3 (see http://www.gnu.org/licenses/lgpl.html)
#
# Website - http://www.pilas-engine.com.ar

import pilas
from pilas.actores import Pizarra

class Mapa(Pizarra):
    "Representa un escenario de bloques que tiene filas y columnas."

    def __init__(self, grilla, x=0, y=0):
        Pizarra.__init__(self, x=x, y=y)
        self.grilla = grilla
        pilas.eventos.inicia_modo_depuracion.conectar(self._cuando_inicia_modo_depuracion)
        pilas.eventos.actualiza_modo_depuracion.conectar(self._cuando_actualiza_modo_depuracion)
        pilas.eventos.sale_modo_depuracion.conectar(self._cuando_sale_modo_depuracion)
        self.figuras = []

    def pintar_bloque(self, fila, columna, indice, solido=True):
        self.grilla.definir_cuadro(indice)

        ancho = self.grilla.cuadro_ancho
        alto = self.grilla.cuadro_alto
        x = columna * ancho
        y = fila * alto

        self.pintar_grilla(self.grilla, x, y)

        if solido:
            figura = pilas.fisica.fisica.crear_cuadrado_estatico(x - 320, 240 -y, ancho)
            self.figuras.append(figura)

    def _cuando_inicia_modo_depuracion(self, evento):
        self.pizarra_depuracion = pilas.actores.Pizarra()

        self._dibujar_rectangulos_de_la_grilla()

    def _cuando_actualiza_modo_depuracion(self, evento):
        pass
        
    def _cuando_sale_modo_depuracion(self, evento):
        self.pizarra_depuracion.eliminar()

    def _dibujar_rectangulos_de_la_grilla(self):
        ancho = self.grilla.cuadro_ancho
        alto = self.grilla.cuadro_alto
        tamano = 8
        self.pizarra_depuracion.definir_color(pilas.colores.rojo)
        self.pizarra_depuracion.deshabilitar_actualizacion_automatica()

        for columna, x in enumerate(range(0, 640, ancho)):
            for fila, y in enumerate(range(0, 480, alto)):
                self.pizarra_depuracion.dibujar_rectangulo(x, y, ancho, alto, pintar=False)
                texto = "(%d, %d)" %(fila, columna)
                self.pizarra_depuracion.escribir(texto, x=x+3, y=y+10, tamano=tamano)

        self.pizarra_depuracion.habilitar_actualizacion_automatica()

    def eliminar(self):

        # Elimina todas las figuras fisicas que ha creado.
        for x in self.figuras:
            pilas.fisica.fisica.eliminar(x)

        Pizarra.eliminar(self)



# -*- encoding: utf-8 -*-
# Pilas engine - A video game framework.
#
# Copyright 2010 - Hugo Ruscitti
# License: LGPLv3 (see http://www.gnu.org/licenses/lgpl.html)
#
# Website - http://www.pilas-engine.com.ar

import pilas
from pilas.actores import Actor
import copy

class Animado(Actor):
    """Representa un actor que tiene asociada una grilla con cuadros de animacion.

    Una de las variantes que introduce este actor es el
    método 'definir_cuadro', que facilita la animación de personajes.
    """

    def __init__(self, grilla, x=0, y=0):
        Actor.__init__(self, x=x, y=y)
        self.animacion = copy.copy(grilla)
        self.definir_cuadro(0)

    def definir_cuadro(self, indice):
        self.animacion.definir_cuadro(indice)
        self.animacion.asignar(self)



# -*- encoding: utf-8 -*-
# Pilas engine - A video game framework.
#
# Copyright 2010 - Hugo Ruscitti
# License: LGPLv3 (see http://www.gnu.org/licenses/lgpl.html)
#
# Website - http://www.pilas-engine.com.ar
import os
import shutil
import interpolaciones
import sys
import subprocess
import math

import pilas


PATH = os.path.dirname(os.path.abspath(__file__))


def cargar_autocompletado():
    "Carga los modulos de python para autocompletar desde la consola interactiva."
    import rlcompleter
    import readline

    readline.parse_and_bind("tab: complete")

def hacer_flotante_la_ventana():
    "Hace flotante la ventana para i3 (el manejador de ventanas que utiliza hugo...)"
    try:
        subprocess.call(['i3-msg', 't'], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    except OSError:
        pass

def es_interpolacion(an_object):
    "Indica si un objeto se comporta como una colisión."

    return isinstance(an_object, interpolaciones.Interpolacion)


def obtener_ruta_al_recurso(ruta):
    """Busca la ruta a un archivo de recursos.

    Los archivos de recursos (como las imagenes) se buscan en varios
    directorios (ver docstring de image.load), así que esta
    función intentará dar con el archivo en cuestión.
    """

    dirs = ['./', os.path.dirname(sys.argv[0]), 'data', PATH, PATH + '/data']


    for x in dirs:
        full_path = os.path.join(x, ruta)
        #DEBUG: print "buscando en: '%s'" %(full_path)

        if os.path.exists(full_path):
            return full_path

    # Si no ha encontrado el archivo lo reporta.
    raise IOError("El archivo '%s' no existe." %(ruta))


def esta_en_sesion_interactiva():
    "Indica si pilas se ha ejecutado desde una consola interactiva de python."
    import sys
    try:
        cursor = sys.ps1
        return True
    except AttributeError:
        try:
            in_ipython = sys.ipcompleter
            return True
        except AttributeError:
            if sys.stdin.__class__.__module__.startswith("idle"):
                return True

    return False

def distancia(a, b):
    "Retorna la distancia entre dos numeros."
    return abs(b - a)

def distancia_entre_dos_puntos((x1, y1), (x2, y2)):
    "Retorna la distancia entre dos puntos en dos dimensiones."
    return math.sqrt(distancia(x1, x2) ** 2 + distancia(y1, y2) ** 2)

def distancia_entre_dos_actores(a, b):
    return distancia_entre_dos_puntos((a.x, a.y), (b.x, b.y))

def colisionan(a, b):
    "Retorna True si dos actores estan en contacto."
    return distancia_entre_dos_actores(a, b) < a.radio_de_colision + b.radio_de_colision

def crear_juego():
    nombre = raw_input("Indica el nombre del juego: ")
    raise "Actualmente no implementado..."
    #shutil.copytree(PATH + "/data/juegobase", nombre)

    print "Se ha creado el directorio '%s'" %(nombre)
    print "Ingresa en el directorio y econtrarás los archivos iniciales del juego."


def interpolable(f):
    "Decorador que se aplica a un metodo para que permita animaciones de interpolaciones."

    def inner(*args, **kwargs):
        value = args[1]

        # Si le indican dos argumentos, el primer sera
        # el valor de la interpolacion y el segundo la
        # velocidad.
        if isinstance(value, tuple) and len(value) == 2:
            duracion = value[1]
            value = value[0]
        else:
            duracion = 1

        if isinstance(value, list):
            value = pilas.interpolar(value, duracion=duracion)
        elif isinstance(value, xrange):
            value = pilas.interpolar(list(value), duracion=duracion)
            
        if es_interpolacion(value):
            value.apply(args[0], function=f.__name__)
        else:
            f(args[0], value, **kwargs)

    return inner

def hacer_coordenada_mundo(x, y):
    return (x + 320, 240 - y)



# -*- encoding: utf-8 -*-
# Pilas engine - A video game framework.
#
# Copyright 2010 - Hugo Ruscitti
# License: LGPLv3 (see http://www.gnu.org/licenses/lgpl.html)
#
# Website - http://www.pilas-engine.com.ar

import pilas
from pilas.actores import Animacion
import math

class Nave(Animacion):
    "Representa una nave que puede disparar."

    def __init__(self, x=0, y=0, velocidad=2):
        self.velocidad = velocidad
        grilla = pilas.imagenes.Grilla("nave.png", 2)
        Animacion.__init__(self, grilla, ciclica=True, x=x, y=y)
        self.radio_de_colision = 20
        self.aprender(pilas.habilidades.PuedeExplotar)
        self.contador_frecuencia_disparo = 0
        self.disparos = []

    def actualizar(self):
        Animacion.actualizar(self)

        if pilas.mundo.control.izquierda:
            self.rotacion -= self.velocidad
        elif pilas.mundo.control.derecha:
            self.rotacion += self.velocidad

        if pilas.mundo.control.arriba:
            self.avanzar()

        self.contador_frecuencia_disparo += 1

        if pilas.mundo.control.boton:
            if self.contador_frecuencia_disparo > 10:
                self.contador_frecuencia_disparo = 0
                self.disparar()
        
        self.eliminar_disparos_innecesarios()

    def eliminar_disparos_innecesarios(self):
        for d in list(self.disparos):
            if d.x < -320 or d.x > 320 or d.y < -240 or d.y > 240:
                d.eliminar()
                self.disparos.remove(d)


    def disparar(self):
        "Hace que la nave dispare."
        disparo_nuevo = pilas.actores.Disparo(self.x, self.y, self.rotacion, 4)
        self.disparos.append(disparo_nuevo)

    def avanzar(self):
        "Hace avanzar la nave en direccion a su angulo."
        rotacion_en_radianes = math.radians(-self.rotacion + 90)
        dx = math.cos(rotacion_en_radianes) * self.velocidad
        dy = math.sin(rotacion_en_radianes) * self.velocidad
        self.x += dx
        self.y += dy

    def definir_enemigos(self, grupo):
        "hace que una nave tenga como enemigos a todos los actores del grupo."
        pilas.colisiones.agregar(self.disparos, grupo, self.hacer_explotar_al_enemigo)

    def hacer_explotar_al_enemigo(self, mi_disparo, el_enemigo):
        "Es el método que se invoca cuando se produce una colisión 'tiro <-> enemigo'"
        mi_disparo.eliminar()
        el_enemigo.eliminar()



# -*- encoding: utf-8 -*-
# pilas engine - a video game framework.
#
# copyright 2010 - hugo ruscitti
# license: lgplv3 (see http://www.gnu.org/licenses/lgpl.html)
#
# website - http://www.pilas-engine.com.ar

import pilas

class ColisionesFisicas(pilas.escenas.Normal):

    def __init__(self):
        pilas.escenas.Normal.__init__(self, pilas.colores.grisoscuro)
        pilas.avisar("Un ejemplo de colisiones")

        pilas.fondos.Pasto()

        m = pilas.actores.Mono()
        m.aprender(pilas.habilidades.Arrastrable)
        m.aprender(pilas.habilidades.ColisionableComoPelota)

        b = pilas.actores.Bomba()
        b.aprender(pilas.habilidades.RebotaComoPelota)

        pilas.atajos.fabricar(pilas.actores.Pelota, 20)



# -*- encoding: utf-8 -*-
# Pilas engine - A video game framework.
#
# Copyright 2010 - Hugo Ruscitti
# License: LGPLv3 (see http://www.gnu.org/licenses/lgpl.html)
#
# Website - http://www.pilas-engine.com.ar

from pilas.actores import Actor

class Ejes(Actor):
    "Representa el eje de coordenadas tomado como sistema de referencia."

    def __init__(self, x=0, y=0):
        Actor.__init__(self, "ejes.png", x=x, y=y)
        self.z = 100
